from django.utils.translation import ugettext_lazy as _

from .classes import CredentialBackend


class CredentialBackendAccessToken(CredentialBackend):
    form_field_widgets = {
        'token': {
            'class': 'django.forms.widgets.PasswordInput',
            'kwargs': {
                'render_value': True
            }
        }
    }
    form_fields = {
        'token': {
            'label': _('Token'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'Generated token value used to make API calls.'
            ), 'kwargs': {
                'max_length': 255
            }, 'required': True
        }
    }
    label = _('Access token')

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Identity'), {
                    'fields': ('token',)
                }
            ),
        )

        return fieldsets


class CredentialBackendUsernamePassword(CredentialBackend):
    form_field_widgets = {
        'password': {
            'class': 'django.forms.widgets.PasswordInput',
            'kwargs': {
                'render_value': True
            }
        }
    }
    form_fields = {
        'username': {
            'label': _('Username'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'Pseudonym used to identify a user.'
            ), 'kwargs': {
                'max_length': 254
            }, 'required': True
        }, 'password': {
            'label': _('Password'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'Character string used to authenticate the user.'
            ), 'kwargs': {
                'max_length': 192
            }, 'required': False
        }
    }
    label = _('Username and password')

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Identity'), {
                    'fields': ('username', 'password')
                }
            ),
        )

        return fieldsets



from django.utils.translation import ugettext_lazy as _

from . import PermissionNamespace

namespace = PermissionNamespace(
    label=_('Permissions'), name='permissions'
)

permission_role_create = namespace.add_permission(
    label=_('Create roles'), name='role_create'
)
permission_role_delete = namespace.add_permission(
    label=_('Delete roles'), name='role_delete'
)
permission_role_edit = namespace.add_permission(
    label=_('Edit roles'), name='role_edit'
)
permission_role_view = namespace.add_permission(
    label=_('View roles'), name='role_view'
)



from mayan.apps.document_states.events import event_workflow_template_edited
from mayan.apps.document_states.permissions import permission_workflow_template_edit
from mayan.apps.document_states.tests.mixins.workflow_template_state_action_mixins import (
    WorkflowTemplateStateActionTestMixin,
    WorkflowTemplateStateActionViewTestMixin
)
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..events import (
    event_document_metadata_added, event_document_metadata_edited,
    event_document_metadata_removed
)
from ..models import MetadataType
from ..workflow_actions import (
    DocumentMetadataAddAction, DocumentMetadataEditAction,
    DocumentMetadataRemoveAction
)

from .literals import (
    DOCUMENT_METADATA_ADD_ACTION_CLASS_PATH,
    DOCUMENT_METADATA_EDIT_ACTION_CLASS_PATH,
    DOCUMENT_METADATA_REMOVE_ACTION_CLASS_PATH, TEST_METADATA_VALUE,
    TEST_METADATA_VALUE_EDITED
)
from .mixins.document_metadata_mixins import DocumentMetadataMixin


class DocumentMetadataActionTestCase(
    DocumentMetadataMixin, WorkflowTemplateStateActionTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_metadata_type()
        self._test_document_type.metadata.create(
            metadata_type=self._test_metadata_type
        )

    def test_document_metadata_add_action(self):
        metadata_count = self._test_document.metadata.count()

        self._clear_events()

        self._execute_workflow_template_state_action(
            klass=DocumentMetadataAddAction, kwargs={
                'metadata_types': MetadataType.objects.all()
            }
        )
        self.assertEqual(
            self._test_document.metadata.count(), metadata_count + 1
        )
        self.assertTrue(
            self._test_document.metadata.filter(
                metadata_type=self._test_metadata_type
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_document)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_metadata_added.id)

    def test_document_metadata_edit_action(self):
        self._create_test_document_metadata()

        metadata_value = self._test_document.metadata.first().value

        self._clear_events()

        self._execute_workflow_template_state_action(
            klass=DocumentMetadataEditAction, kwargs={
                'metadata_type': self._test_metadata_type.pk,
                'value': TEST_METADATA_VALUE_EDITED
            }
        )
        self.assertNotEqual(
            metadata_value, self._test_document.metadata.first().value
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_document)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_metadata_edited.id)

    def test_document_metadata_remove_action(self):
        self._create_test_document_metadata()

        metadata_count = self._test_document.metadata.count()

        self._clear_events()

        self._execute_workflow_template_state_action(
            klass=DocumentMetadataRemoveAction, kwargs={
                'metadata_types': MetadataType.objects.all()
            }
        )
        self.assertEqual(
            self._test_document.metadata.count(), metadata_count - 1
        )
        self.assertFalse(
            self._test_document.metadata.filter(
                metadata_type=self._test_metadata_type
            ).exists()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_document)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_metadata_removed.id)


class DocumentMetadataAddActionViewTestCase(
    DocumentMetadataMixin, WorkflowTemplateStateActionViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_create_test_workflow_template = False
    auto_create_test_workflow_template_state = False
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_metadata_type()
        self._test_document_type.metadata.create(
            metadata_type=self._test_metadata_type
        )
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

    def test_document_metadata_add_action_create_view(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_create_post_view(
            backend_path=DOCUMENT_METADATA_ADD_ACTION_CLASS_PATH,
            extra_data={'metadata_types': self._test_metadata_type.pk}
        )
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_document_metadata_add_action_edit_get_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_ADD_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_get_view()
        self.assertEqual(response.status_code, 200)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_metadata_add_action_edit_post_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_ADD_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_post_view(
            extra_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertNotEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)


class DocumentMetadataEditActionViewTestCase(
    DocumentMetadataMixin, WorkflowTemplateStateActionViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_create_test_workflow_template = False
    auto_create_test_workflow_template_state = False
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_metadata_type()
        self._test_document_type.metadata.create(
            metadata_type=self._test_metadata_type
        )
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

    def test_document_metadata_edit_action_create_view(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_create_post_view(
            backend_path=DOCUMENT_METADATA_EDIT_ACTION_CLASS_PATH,
            extra_data={
                'metadata_type': self._test_metadata_type.pk,
                'value_template': TEST_METADATA_VALUE
            }
        )
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_document_metadata_edit_action_edit_get_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_EDIT_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                # ~ 'metadata_types': self._test_metadata_type.pk
                'metadata_type': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_get_view()
        self.assertEqual(response.status_code, 200)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_metadata_edit_action_edit_post_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_EDIT_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                # ~ 'metadata_types': self._test_metadata_type.pk
                'metadata_type': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_post_view(
            extra_data={
                'metadata_type': self._test_metadata_type.pk,
                'value_template': TEST_METADATA_VALUE
            }
        )
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertNotEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)


class DocumentMetadataRemoveActionViewTestCase(
    DocumentMetadataMixin, WorkflowTemplateStateActionViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_create_test_workflow_template = False
    auto_create_test_workflow_template_state = False
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_metadata_type()
        self._test_document_type.metadata.create(
            metadata_type=self._test_metadata_type
        )
        self._test_workflow_template.document_types.add(
            self._test_document_type
        )

    def test_document_metadata_remove_action_create_view(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_create_post_view(
            backend_path=DOCUMENT_METADATA_REMOVE_ACTION_CLASS_PATH,
            extra_data={'metadata_types': self._test_metadata_type.pk}
        )
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_document_metadata_remove_action_edit_get_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_REMOVE_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_get_view()
        self.assertEqual(response.status_code, 200)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_metadata_remove_action_edit_post_view(self):
        self._test_workflow_template_state_action_path = DOCUMENT_METADATA_REMOVE_ACTION_CLASS_PATH

        self._create_test_workflow_template_state_action(
            extra_backend_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_post_view(
            extra_data={
                'metadata_types': self._test_metadata_type.pk
            }
        )
        self.assertEqual(response.status_code, 302)

        self._test_workflow_template_state_action.refresh_from_db()

        self.assertNotEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0005_auto_20150706_1832')
    ]

    operations = [
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='language',
            field=models.CharField(
                max_length=8, verbose_name='Language', choices=[
                    ('ar', 'Arabic'), ('bg', 'Bulgarian'),
                    ('bs', 'Bosnian (Bosnia and Herzegovina)'),
                    ('da', 'Danish'), ('de', 'German (Germany)'),
                    ('en', 'English'), ('es', 'Spanish'), ('fa', 'Persian'),
                    ('fr', 'French'), ('hu', 'Hungarian'), ('hr', 'Croatian'),
                    ('id', 'Indonesian'), ('it', 'Italian'),
                    ('nl', 'Dutch (Netherlands)'), ('pl', 'Polish'),
                    ('pt', 'Portuguese'), ('pt-br', 'Portuguese (Brazil)'),
                    ('ro', 'Romanian (Romania)'), ('ru', 'Russian'),
                    ('sl', 'Slovenian'), ('tr', 'Turkish'),
                    ('vi', 'Vietnamese (Viet Nam)'),
                    ('zh-cn', 'Chinese (China)')
                ]
            )
        )
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_object, menu_return, menu_tools
from mayan.apps.navigation.classes import SourceColumn

from .classes import Dashboard
from .links import link_dashboard_detail, link_dashboard_list


class DashboardsApp(MayanAppConfig):
    app_namespace = 'dashboards'
    app_url = 'dashboards'
    has_rest_api = False
    has_static_media = True
    has_tests = True
    name = 'mayan.apps.dashboards'
    verbose_name = _('Dashboards')

    def ready(self):
        super().ready()

        SourceColumn(
            func=lambda context: len(context['object'].widgets),
            label=_('Widgets'), include_label=True,
            source=Dashboard
        )
        SourceColumn(
            attribute='label', label=_('Label'), is_identifier=True,
            source=Dashboard
        )

        menu_object.bind_links(
            links=(link_dashboard_detail,), sources=(Dashboard,)
        )
        menu_return.bind_links(
            links=(link_dashboard_list,), sources=(
                Dashboard, 'dashboards:dashboard_list',
            )
        )
        menu_tools.bind_links(
            links=(link_dashboard_list,)
        )



from django.apps import apps
from django.conf import settings
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_tools
from mayan.apps.organizations.settings import setting_organization_url_base_path

from .links import (
    link_api, link_api_documentation, link_api_documentation_redoc
)


class RESTAPIApp(MayanAppConfig):
    app_url = 'api'
    app_namespace = 'rest_api'
    has_tests = True
    name = 'mayan.apps.rest_api'
    static_media_ignore_patterns = (
        'rest_framework/docs/*', 'rest_framework/img/glyphicons*',
    )
    verbose_name = _('REST API')

    def ready(self):
        super().ready()
        from .urls import api_version_urls

        installation_base_url = setting_organization_url_base_path.value
        if installation_base_url:
            installation_base_url = '/{}'.format(installation_base_url)
        else:
            installation_base_url = ''

        settings.STRONGHOLD_PUBLIC_URLS += (
            r'^{}/{}/'.format(installation_base_url, self.app_url),
        )

        settings.STRONGHOLD_PUBLIC_URLS += (
            r'^{}/{}/.+$'.format(installation_base_url, self.app_url),
        )
        menu_tools.bind_links(
            links=(
                link_api, link_api_documentation,
                link_api_documentation_redoc
            )
        )

        for app in apps.get_app_configs():
            if getattr(app, 'has_rest_api', False):
                app_api_urls = import_string(
                    dotted_path='{}.urls.api_urls'.format(app.name)
                )
                api_version_urls.extend(app_api_urls)



from django.db import migrations


def code_copy_locales(apps, schema_editor):
    UserLocaleProfile = apps.get_model(
        app_label='common', model_name='UserLocaleProfile'
    )
    UserLocaleProfileNew = apps.get_model(
        app_label='locales', model_name='UserLocaleProfileNew'
    )

    for locale_profile in UserLocaleProfile.objects.using(alias=schema_editor.connection.alias):
        UserLocaleProfileNew.objects.create(
            user=locale_profile.user,
            timezone=locale_profile.timezone,
            language=locale_profile.language
        )


def code_copy_locales_reverse(apps, schema_editor):
    UserLocaleProfile = apps.get_model(
        app_label='common', model_name='UserLocaleProfile'
    )
    UserLocaleProfileNew = apps.get_model(
        app_label='locales', model_name='UserLocaleProfileNew'
    )

    for locale_profile in UserLocaleProfileNew.objects.using(alias=schema_editor.connection.alias):
        UserLocaleProfile.objects.create(
            user=locale_profile.user,
            timezone=locale_profile.timezone,
            language=locale_profile.language
        )


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0015_auto_20200501_0631'),
        ('locales', '0001_initial')
    ]

    operations = [
        migrations.RunPython(
            code=code_copy_locales,
            reverse_code=code_copy_locales_reverse,
        ),
    ]

    run_before = [
        ('common', '0018_delete_userlocaleprofile'),
    ]



from mayan.apps.documents.tests.mixins.document_mixins import DocumentTypeTestMixin
from mayan.apps.testing.tests.mixins import TestMixinObjectCreationTrack

from ...models import MetadataType

from ..literals import (
    TEST_METADATA_TYPE_LABEL, TEST_METADATA_TYPE_LABEL_EDITED,
    TEST_METADATA_TYPE_NAME, TEST_METADATA_TYPE_NAME_EDITED
)


class MetadataTypeTestMixin(
    DocumentTypeTestMixin, TestMixinObjectCreationTrack
):
    _test_object_model = MetadataType
    _test_object_name = '_test_metadata_type'
    auto_add_test_metadata_type_to_test_document_type = True
    auto_create_test_metadata_type = False
    auto_create_test_metadata_type_is_required = False

    def setUp(self):
        super().setUp()
        self._test_metadata_type_list = []
        self._test_document_type_metadata_type_relationships = []

        if self.auto_create_test_metadata_type:
            self._create_test_metadata_type(
                add_test_document_type=self.auto_add_test_metadata_type_to_test_document_type,
                required=self.auto_create_test_metadata_type_is_required
            )

    def _get_test_metadata_type_queryset(self):
        return MetadataType.objects.filter(
            pk__in=[
                metadata_type.pk for metadata_type in self._test_metadata_type_list
            ]
        )

    def _create_test_metadata_type(
        self, add_test_document_type=False, extra_kwargs=None, required=False
    ):
        total_test_metadata_type_list = len(self._test_metadata_type_list)
        name = '{}_{}'.format(
            TEST_METADATA_TYPE_NAME, total_test_metadata_type_list
        )
        label = '{}_{}'.format(
            TEST_METADATA_TYPE_LABEL, total_test_metadata_type_list
        )

        kwargs = {'name': name, 'label': label}

        if extra_kwargs:
            kwargs.update(extra_kwargs)

        self._test_metadata_type = MetadataType.objects.create(**kwargs)
        self._test_metadata_type_list.append(self._test_metadata_type)

        if add_test_document_type:
            self._test_document_type_metadata_type_relationships.append(
                self._test_document_type.metadata.create(
                    metadata_type=self._test_metadata_type, required=required
                )
            )


class MetadataTypeAPIViewTestMixin(MetadataTypeTestMixin):
    def _request_test_metadata_type_create_api_view(self):
        self._test_object_track()

        response = self.post(
            viewname='rest_api:metadatatype-list', data={
                'name': 'test_metadata_type', 'label': 'test metadata type'
            }
        )

        self._test_object_set()

        return response

    def _request_test_metadata_type_delete_api_view(self):
        return self.delete(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}
        )

    def _request_test_metadata_type_detail_api_view(self):
        return self.get(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}
        )

    def _request_test_metadata_type_edit_api_view_via_patch(self):
        return self.patch(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}, data={
                'label': '{} edited'.format(self._test_metadata_type.label),
                'name': '{}_edited'.format(self._test_metadata_type.name)
            }
        )

    def _request_test_metadata_type_edit_api_view_via_put(self):
        return self.put(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}, data={
                'label': '{} edited'.format(self._test_metadata_type.label),
                'name': '{}_edited'.format(self._test_metadata_type.name)
            }
        )

    def _request_test_metadata_type_list_api_view(self):
        return self.get(viewname='rest_api:metadatatype-list')


class MetadataTypeViewTestMixin(MetadataTypeTestMixin):
    def _request_test_document_type_relationship_delete_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:document_type_metadata_type_relationship',
            kwargs={'document_type_id': self._test_document_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'none'
            }
        )

    def _request_test_document_type_relationship_edit_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:document_type_metadata_type_relationship',
            kwargs={'document_type_id': self._test_document_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'required'
            }
        )

    def _request_test_metadata_type_create_view(self):
        self._test_object_track()

        response = self.post(
            viewname='metadata:metadata_type_create', data={
                'label': 'test metadata type', 'name': 'test_metadata_type'
            }
        )

        self._test_object_set()

        return response

    def _request_test_metadata_type_single_delete_view(self):
        return self.post(
            viewname='metadata:metadata_type_single_delete', kwargs={
                'metadata_type_id': self._test_metadata_type.pk
            }
        )

    def _request_test_metadata_type_multiple_delete_view(self):
        return self.post(
            viewname='metadata:metadata_type_multiple_delete', data={
                'id_list': self._test_metadata_type.pk
            }
        )

    def _request_test_metadata_type_edit_view(self):
        return self.post(
            viewname='metadata:metadata_type_edit', kwargs={
                'metadata_type_id': self._test_metadata_type.pk
            }, data={
                'label': TEST_METADATA_TYPE_LABEL_EDITED,
                'name': TEST_METADATA_TYPE_NAME_EDITED
            }
        )

    def _request_metadata_type_list_view(self):
        return self.get(viewname='metadata:metadata_type_list')

    def _request_test_metadata_type_document_type_relationship_delete_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:metadata_type_document_type_relationship',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'none'
            }
        )

    def _request_test_metadata_type_document_type_relationship_edit_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:metadata_type_document_type_relationship',
            kwargs={'metadata_type_id': self._test_metadata_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'required'
            }
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import factory_condition_queryset_access

from .icons import (
    icon_document_tag_list, icon_document_tag_multiple_attach,
    icon_document_tag_multiple_remove, icon_tag_create,
    icon_tag_document_list, icon_tag_edit, icon_tag_list,
    icon_tag_multiple_delete, icon_tag_single_delete
)
from .permissions import (
    permission_tag_attach, permission_tag_create, permission_tag_delete,
    permission_tag_edit, permission_tag_remove, permission_tag_view
)

link_document_multiple_tag_multiple_attach = Link(
    icon=icon_document_tag_multiple_attach, text=_('Attach tags'),
    view='tags:multiple_documents_tag_attach'
)
link_document_multiple_tag_multiple_remove = Link(
    icon=icon_document_tag_multiple_remove, text=_('Remove tag'),
    view='tags:multiple_documents_selection_tag_remove'
)
link_document_tag_list = Link(
    args='resolved_object.pk', icon=icon_document_tag_list,
    permissions=(permission_tag_view,), text=_('Tags'),
    view='tags:document_tag_list'
)
link_document_tag_multiple_remove = Link(
    args='object.id', icon=icon_document_tag_multiple_remove,
    permissions=(permission_tag_remove,), text=_('Remove tags'),
    view='tags:single_document_multiple_tag_remove'
)
link_document_tag_multiple_attach = Link(
    args='object.pk', icon=icon_document_tag_multiple_attach,
    permissions=(permission_tag_attach,), text=_('Attach tags'),
    view='tags:tag_attach'
)

link_tag_create = Link(
    icon=icon_tag_create, permissions=(permission_tag_create,),
    text=_('Create new tag'), view='tags:tag_create'
)
link_tag_single_delete = Link(
    args='object.id', icon=icon_tag_single_delete,
    permissions=(permission_tag_delete,), tags='dangerous',
    text=_('Delete'), view='tags:tag_single_delete'
)
link_tag_multiple_delete = Link(
    icon=icon_tag_multiple_delete, text=_('Delete'),
    view='tags:tag_multiple_delete'
)
link_tag_edit = Link(
    args='object.id', icon=icon_tag_edit,
    permissions=(permission_tag_edit,), text=_('Edit'),
    view='tags:tag_edit'
)
link_tag_list = Link(
    condition=factory_condition_queryset_access(
        app_label='tags', model_name='Tag',
        object_permission=permission_tag_view,
    ), icon=icon_tag_list,
    text=_('All'), view='tags:tag_list'
)
link_tag_document_list = Link(
    args='object.id', icon=icon_tag_document_list,
    text=('Documents'), view='tags:tag_document_list'
)



from django.db.models.signals import post_migrate
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import (
    permission_acl_edit, permission_acl_view
)
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_multi_item, menu_object, menu_return, menu_secondary, menu_setup
)
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.navigation.classes import SourceColumn

from .events import event_asset_edited
from .handlers import handler_create_asset_cache
from .links import (
    link_asset_create, link_asset_multiple_delete, link_asset_single_delete,
    link_asset_edit, link_asset_list, link_asset_setup,
    link_transformation_delete, link_transformation_edit,
    link_transformation_select
)
from .permissions import (
    permission_asset_delete, permission_asset_edit, permission_asset_view
)


class ConverterApp(MayanAppConfig):
    app_namespace = 'converter'
    app_url = 'converter'
    has_rest_api = True
    has_static_media = True
    has_tests = True
    name = 'mayan.apps.converter'
    verbose_name = _('Converter')

    def ready(self):
        super().ready()

        Asset = self.get_model(model_name='Asset')
        LayerTransformation = self.get_model(
            model_name='LayerTransformation'
        )

        EventModelRegistry.register(model=Asset)

        ModelEventType.register(
            model=Asset, event_types=(
                event_asset_edited,
            )
        )

        ModelPermission.register(
            model=Asset, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_asset_delete, permission_asset_edit,
                permission_asset_view
            )
        )

        ModelPermission.register_inheritance(
            model=LayerTransformation,
            related='object_layer__content_object'
        )

        SourceColumn(
            attribute='label', is_identifier=True,
            is_object_absolute_url=True, is_sortable=True, source=Asset
        )
        SourceColumn(
            attribute='internal_name', include_label=True,
            is_object_absolute_url=True, is_sortable=True, source=Asset
        )

        SourceColumn(
            attribute='order', is_identifier=True, is_sortable=True,
            source=LayerTransformation
        )
        SourceColumn(
            func=lambda context: str(
                context['object']
            ), include_label=True, label=_('Transformation'),
            source=LayerTransformation
        )
        SourceColumn(
            attribute='get_arguments_column', include_label=True,
            source=LayerTransformation
        )

        menu_multi_item.bind_links(
            links=(link_asset_multiple_delete,), sources=(Asset,)
        )
        menu_object.bind_links(
            links=(
                link_asset_single_delete, link_asset_edit
            ), sources=(Asset,)
        )
        menu_return.bind_links(
            links=(link_asset_list,),
            sources=(
                Asset, 'converter:asset_list', 'converter:asset_create',
                'converter:asset_multiple_delete'
            )
        )
        menu_secondary.bind_links(
            links=(link_asset_create,),
            sources=(
                Asset, 'converter:asset_list', 'converter:asset_create',
                'converter:asset_multiple_delete'
            )
        )
        menu_setup.bind_links(
            links=(link_asset_setup,)
        )

        menu_object.bind_links(
            links=(link_transformation_edit, link_transformation_delete),
            sources=(LayerTransformation,)
        )
        menu_secondary.bind_links(
            links=(link_transformation_select,),
            sources=(LayerTransformation,)
        )
        menu_secondary.bind_links(
            links=(link_transformation_select,),
            sources=(
                'converter:transformation_create',
                'converter:transformation_list'
            )
        )

        post_migrate.connect(
            dispatch_uid='converter_handler_create_asset_cache',
            receiver=handler_create_asset_cache
        )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('web_links', '0004_make_labes_unique')
    ]

    operations = [
        migrations.AlterField(
            model_name='weblink',
            name='label',
            field=models.CharField(
                db_index=True, help_text='A short text describing the web '
                'link.', max_length=96, unique=True, verbose_name='Label'
            )
        )
    ]



from mayan.apps.common.tests.mixins import PropertyHelperTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from .mixins.document_metadata_mixins import DocumentMetadataMixin


class DocumentPropertyHelperTestCase(
    DocumentMetadataMixin, PropertyHelperTestMixin, BaseTestCase
):
    auto_create_test_metadata_type = True

    def test_basic(self):
        self._create_test_document_metadata()

        self._clear_events()

        self.assertEqual(
            getattr(
                self._test_document.metadata_value_of,
                self._test_document_metadata.metadata_type.name
            ), self._test_document_metadata.value
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_no_source_metadata(self):
        self._create_test_document_metadata()

        self._test_document_metadata.delete()

        self._clear_events()

        self.assertEqual(
            getattr(
                self._test_document.metadata_value_of,
                self._test_document_metadata.metadata_type.name
            ), None
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_value_empty(self):
        self._test_document_metadata_value = ''
        self._create_test_document_metadata()

        self._clear_events()

        self.assertEqual(
            getattr(
                self._test_document.metadata_value_of,
                self._test_document_metadata.metadata_type.name
            ), ''
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_value_none(self):
        self._test_document_metadata_value = None
        self._create_test_document_metadata()

        self._clear_events()

        self.assertEqual(
            getattr(
                self._test_document.metadata_value_of,
                self._test_document_metadata.metadata_type.name
            ), None
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_wrong_source_metadata_name(self):
        self._create_test_document_metadata()

        self._clear_events()

        self.assertEqual(
            getattr(
                self._test_document.metadata_value_of, 'invalid'
            ), None
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from unittest import mock, skip

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.tags.tests.mixins import TagTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from ..exceptions import DynamicSearchBackendException
from ..search_query_types import QueryTypeExact

from .mixins.backend_mixins import BackendSearchTestMixin
from .mixins.backend_query_type_mixins import BackendFieldTypeQueryTypeTestCaseMixin
from .mixins.backend_search_field_mixins import BackendSearchFieldTestCaseMixin
from .mixins.base import SearchTestMixin, TestSearchObjectSimpleTestMixin


class DjangoSearchBackendSearchFieldTestCase(
    BackendSearchFieldTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.django.DjangoSearchBackend'


class DjangoSearchBackendFieldTypeQueryTypeTestCase(
    BackendFieldTypeQueryTypeTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.django.DjangoSearchBackend'

    @skip(reason='Backend does not support the feature.')
    def test_search_field_type_char_search_exact_accent(self):
        """
        This backend does not require indexing which is required
        for this feature to work.
        """

    @skip(reason='Backend does not support the feature.')
    def test_search_field_type_char_search_fuzzy(self):
        """
        This query type is emulated and does not return the same results
        as backends support this natively.
        """

    @skip(reason='Backend does not support the feature.')
    def test_search_field_type_text_search_fuzzy(self):
        """
        This query type is emulated and does not return the same results
        as backends support this natively.
        """


@skip(reason='Skip until a Mock ElasticSearch server class is added.')
class ElasticSearchBackendIndexingTestCase(
    BackendSearchTestMixin, TestSearchObjectSimpleTestMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.elasticsearch.ElasticSearchBackend'

    def test_search_without_indexes(self):
        self._test_search_backend.tear_down()

        with self.assertRaises(expected_exception=DynamicSearchBackendException):
            self._do_backend_search(
                field_name='char',
                query_type=QueryTypeExact,
                value=self._test_object.char,
                _skip_refresh=True,
            )


@skip(reason='Skip until a Mock ElasticSearch server class is added.')
class ElasticSearchBackendSearchFieldTestCase(
    BackendSearchFieldTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.elasticsearch.ElasticSearchBackend'


@skip(reason='Skip until a Mock ElasticSearch server class is added.')
class ElasticSearchBackendFieldTypeQueryTypeTestCase(
    BackendFieldTypeQueryTypeTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.elasticsearch.ElasticSearchBackend'


class WhooshSearchBackendSearchFieldTestCase(
    BackendSearchFieldTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.whoosh.WhooshSearchBackend'


class WhooshSearchBackendFieldTypeQueryTypeTestCase(
    BackendFieldTypeQueryTypeTestCaseMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.whoosh.WhooshSearchBackend'


class WhooshSearchBackendSpecificTestCase(
    BackendSearchTestMixin, TestSearchObjectSimpleTestMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.backends.whoosh.WhooshSearchBackend'

    def test_whoosh_datetime_search_raw_parsed_date_human_today(self):
        id_list = self._do_backend_search(
            field_name='datetime',
            is_raw_value=True,
            query_type=QueryTypeExact,
            value='today'
        )

        self.assertEqual(
            len(id_list), 1
        )
        self.assertTrue(self._test_object.id in id_list)

    def test_whoosh_datetime_search_raw_parsed_date_human_range(self):
        id_list = self._do_backend_search(
            field_name='datetime',
            is_raw_value=True,
            query_type=QueryTypeExact,
            value='[\'last tuesday\' to \'next friday\']'
        )

        self.assertEqual(
            len(id_list), 1
        )
        self.assertTrue(self._test_object.id in id_list)

    def test_whoosh_datetime_search_raw_parsed_date_numeric_range(self):
        id_list = self._do_backend_search(
            field_name='datetime',
            is_raw_value=True,
            query_type=QueryTypeExact,
            value='[\'{}\' to \'{}\']'.format(
                self._test_object.datetime.year - 1,
                self._test_object.datetime.year + 1
            )
        )

        self.assertEqual(
            len(id_list), 1
        )
        self.assertTrue(self._test_object.id in id_list)

    def test_whoosh_integer_search_raw_parsed_numeric_range(self):
        id_list = self._do_backend_search(
            field_name='integer',
            is_raw_value=True,
            query_type=QueryTypeExact,
            value='[\'{}\' to \'{}\']'.format(
                self._test_object.integer - 1,
                self._test_object.integer + 1
            )
        )

        self.assertEqual(
            len(id_list), 1
        )
        self.assertTrue(self._test_object.id in id_list)


class SearchUpdatePropagationTestCase(
    DocumentTestMixin, SearchTestMixin, TagTestMixin, BaseTestCase
):
    _test_search_backend_path = 'mayan.apps.dynamic_search.tests.backends.DummySearchBackend'
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()

        self._create_test_document_stub()
        self._create_test_document_stub()
        self._create_test_document_stub()
        self._create_test_tag()

    @mock.patch(target='mayan.apps.dynamic_search.tests.backends.DummySearchBackend.index_instance')
    def test_m2m_add_propagation(self, mocked_index_instance):
        self._test_tag._attach_to(
            document=self._test_documents[0]
        )
        self.assertEqual(mocked_index_instance.call_count, 2)
        self.assertEqual(
            mocked_index_instance.call_args_list[0].kwargs['instance'],
            self._test_documents[0]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[1].kwargs['instance'],
            self._test_tag_list[0]
        )

        mocked_index_instance.reset_mock()

        self._test_tag._attach_to(
            document=self._test_documents[1]
        )
        self.assertEqual(mocked_index_instance.call_count, 2)
        self.assertEqual(
            mocked_index_instance.call_args_list[0].kwargs['instance'],
            self._test_documents[1]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[1].kwargs['instance'],
            self._test_tag_list[0]
        )

        mocked_index_instance.reset_mock()

        self._test_tag._attach_to(
            document=self._test_documents[2]
        )
        self.assertEqual(mocked_index_instance.call_count, 2)
        self.assertEqual(
            mocked_index_instance.call_args_list[0].kwargs['instance'],
            self._test_documents[2]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[1].kwargs['instance'],
            self._test_tag_list[0]
        )

        mocked_index_instance.reset_mock()

        self._test_tag.label = 'edited'
        self._test_tag.save()
        self.assertEqual(mocked_index_instance.call_count, 4)
        self.assertEqual(
            mocked_index_instance.call_args_list[0].kwargs['instance'],
            self._test_documents[0]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[1].kwargs['instance'],
            self._test_documents[1]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[2].kwargs['instance'],
            self._test_documents[2]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[3].kwargs['instance'],
            self._test_tag_list[0]
        )

        mocked_index_instance.reset_mock()

        self._test_tag._remove_from(
            document=self._test_documents[0]
        )
        self.assertEqual(mocked_index_instance.call_count, 2)
        self.assertEqual(
            mocked_index_instance.call_args_list[0].kwargs['instance'],
            self._test_documents[0]
        )
        self.assertEqual(
            mocked_index_instance.call_args_list[1].kwargs['instance'],
            self._test_tag_list[0]
        )



from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.backends.model_mixins import BackendModelMixin
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import EventManagerSave

from .classes import NullBackend
from .events import event_profile_created, event_profile_edited
from .managers import UserMailerManager
from .model_mixins import UserMailerBusinessLogicMixin


class UserMailer(
    BackendModelMixin, UserMailerBusinessLogicMixin, models.Model
):
    """
    This model is used to create mailing profiles that can be used from inside
    the system. These profiles differ from the system mailing profile in that
    they can be created at runtime and can be assigned ACLs to restrict
    their use.
    """
    _backend_model_null_backend = NullBackend

    label = models.CharField(
        help_text=_('A short text describing the mailing profile.'),
        max_length=128, unique=True, verbose_name=_('Label')
    )
    default = models.BooleanField(
        default=True, help_text=_(
            'If default, this mailing profile will be pre-selected on the '
            'document mailing form.'
        ), verbose_name=_('Default')
    )
    enabled = models.BooleanField(
        default=True, verbose_name=_('Enabled')
    )

    objects = UserMailerManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Mailing profile')
        verbose_name_plural = _('Mailing profiles')

    def __str__(self):
        return self.label

    def get_absolute_url(self):
        return reverse(viewname='mailer:user_mailer_list')

    def natural_key(self):
        return (self.label,)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_profile_created,
            'target': 'self'
        },
        edited={
            'event': event_profile_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        if self.default:
            UserMailer.objects.select_for_update().exclude(
                pk=self.pk
            ).update(default=False)

        return super().save(*args, **kwargs)



from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.converter.fields import ImageField
from mayan.apps.views.forms import FilteredSelectionForm

from ..models import Workflow


class WorkflowTemplateForm(forms.ModelForm):
    class Meta:
        fields = ('label', 'internal_name', 'auto_launch')
        model = Workflow


class WorkflowTemplateSelectionForm(FilteredSelectionForm):
    class Meta:
        allow_multiple = True
        field_name = 'workflows'
        label = _('Workflows')
        required = False
        widget_attributes = {'class': 'select2'}


class WorkflowTemplatePreviewForm(forms.Form):
    workflow = ImageField(
        image_alt_text=_('Workflow template preview image')
    )

    def __init__(self, instance=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['workflow'].initial = instance



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import (
    DEFAULT_VIEWS_PAGINATE_BY, DEFAULT_VIEWS_PAGING_ARGUMENT,
    DEFAULT_VIEWS_SHOW_DROPZONE_SUBMIT_BUTTON
)

namespace = SettingNamespace(
    label=_('Views'), name='views'
)

setting_paginate_by = namespace.add_setting(
    default=DEFAULT_VIEWS_PAGINATE_BY, global_name='VIEWS_PAGINATE_BY',
    help_text=_(
        'The number objects that will be displayed per page.'
    )
)
setting_paging_argument = namespace.add_setting(
    default=DEFAULT_VIEWS_PAGING_ARGUMENT,
    global_name='VIEWS_PAGING_ARGUMENT', help_text=_(
        'A string specifying the name to use for the paging parameter.'
    )
)
setting_show_dropzone_submit_button = namespace.add_setting(
    default=DEFAULT_VIEWS_SHOW_DROPZONE_SUBMIT_BUTTON,
    global_name='VIEWS_SHOW_DROPZONE_SUBMIT_BUTTON', help_text=_(
        'Display a submit button and wait for the submit button to be '
        'pressed before processing up the upload.'
    )
)



from django.db import models


class WorkflowManager(models.Manager):
    def launch_for(self, document, user=None):
        for workflow_template in document.document_type.workflows.all():
            if workflow_template.auto_launch:
                workflow_template.launch_for(document=document, user=user)


class ValidWorkflowInstanceManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(document__in_trash=False)



import json
import logging

from django.apps import apps
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.class_mixins import AppsModuleLoaderMixin

from .exceptions import QuotaExceeded
from .handlers import handler_process_quota_signal

__all__ = ('QuotaBackend',)
logger = logging.getLogger(name=__name__)


class QuotaBackendMetaclass(type):
    _registry = {}

    def __new__(mcs, name, bases, attrs):
        new_class = super().__new__(
            mcs, name, bases, attrs
        )
        if not new_class.__module__ == 'mayan.apps.quotas.classes':
            mcs._registry[
                '{}.{}'.format(new_class.__module__, name)
            ] = new_class
            new_class.id = '{}.{}'.format(new_class.__module__, name)

        return new_class


class QuotaBackendBase(AppsModuleLoaderMixin):
    """
    Base class for the mailing backends. This class is mainly a wrapper
    for other Django backends that adds a few metadata to specify the
    fields it needs to be instantiated at runtime.

    The fields attribute is a list of dictionaries with the format:
    {
        'name': ''  # Field internal name
        'label': ''  # Label to show to users
        'class': ''  # Field class to use. Field classes are Python dot
                       paths to Django's form fields.
        'initial': ''  # Field initial value
        'default': ''  # Default value.
    }
    """
    fields = {}
    signal = None
    widgets = {}


class QuotaBackend(QuotaBackendBase, metaclass=QuotaBackendMetaclass):
    _loader_module_name = 'quota_backends'

    @staticmethod
    def _queryset_to_text_list(queryset):
        return ','.join(
            list(
                map(str, queryset)
            )
        ) or _('none')

    @staticmethod
    def connect_signals():
        for backend in QuotaBackend.get_all():
            if backend.signal:
                backend.signal.connect(
                    dispatch_uid='quotas_handler_process_signal',
                    receiver=handler_process_quota_signal,
                    sender=backend.sender
                )

    @classmethod
    def _initialize(cls):
        """
        Allow the quota backend to run code when the app initializes.
        """

    @classmethod
    def create(cls, **kwargs):
        Quota = apps.get_model(app_label='quotas', model_name='Quota')

        return Quota.objects.create(
            backend_data=json.dumps(obj=kwargs),
            backend_path=cls.get_dotted_path()
        )

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return sorted(
            cls._registry.values(), key=lambda x: x.label
        )

    @classmethod
    def get_choices(cls):
        return [
            (
                backend.id, backend.label
            ) for backend in QuotaBackend.get_all()
        ]

    @classmethod
    def get_dotted_path(cls):
        return '{}.{}'.format(cls.__module__, cls.__name__)

    @classmethod
    def get_fields(cls):
        return cls.fields

    @classmethod
    def get_field_order(cls):
        return cls.field_order

    @classmethod
    def get_instances(cls):
        Quota = apps.get_model(app_label='quotas', model_name='Quota')

        return Quota.objects.filter(
            backend_path=cls.get_dotted_path()
        )

    @classmethod
    def get_widgets(cls):
        return cls.widgets

    @classmethod
    def load_modules(cls):
        super().load_modules()

        for backend in QuotaBackend.get_all():
            backend._initialize()

        QuotaBackend.connect_signals()

    def _allowed_filter_display(self):
        return ''

    def filters(self):
        result = self.get_filter_text()

        return format_lazy(
            ', '.join(
                ['{}'] * len(result)
            ), *result.values()
        )

    def get_filter_text(self):
        return {
            'limit': self._allowed_filter_display()
        }

    def process(self, **kwargs):
        if self._usage() >= self._allowed():
            raise QuotaExceeded(self.error_message)

    def usage(self):
        return _('Does not apply')


class NullBackend(QuotaBackend):
    label = _('Null backend')
    signal = None

    def display(self):
        return _('Null backend')



from django.db import migrations, models

import mayan.apps.common.validators


class Migration(migrations.Migration):
    dependencies = [
        ('converter', '0018_asset')
    ]

    operations = [
        migrations.AlterField(
            model_name='layertransformation',
            name='arguments',
            field=models.TextField(
                blank=True, help_text='Enter the arguments for the '
                'transformation as a YAML dictionary. ie: {"degrees": 180}',
                validators=[
                    mayan.apps.common.validators.YAMLValidator()
                ],
                verbose_name='Arguments'
            ),
        )
    ]



import logging

from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from model_utils.managers import InheritanceManager

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.django_gpg.exceptions import VerificationError
from mayan.apps.django_gpg.models import Key
from mayan.apps.documents.models.document_file_models import DocumentFile
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import (
    EventManagerMethodAfter, EventManagerSave
)
from mayan.apps.storage.classes import DefinedStorageLazy

from .events import (
    event_detached_signature_deleted, event_detached_signature_uploaded
)
from .literals import STORAGE_NAME_DOCUMENT_SIGNATURES_DETACHED_SIGNATURE
from .managers import DetachedSignatureManager, EmbeddedSignatureManager
from .model_mixins import SignatureBaseModelBusinessLogicMixin
from .utils import upload_to

logger = logging.getLogger(name=__name__)


class SignatureBaseModel(SignatureBaseModelBusinessLogicMixin, models.Model):
    """
    Fields:
    * key_id - Key Identifier - This is what identifies uniquely a key. Not
    two keys in the world have the same Key ID. The Key ID is also used to
    locate a key in the key servers: http://pgp.mit.edu
    * signature_id - Signature ID - Every time a key is used to sign something
    it will generate a unique signature ID. No two signature IDs are the same,
    even when using the same key.
    """
    document_file = models.ForeignKey(
        editable=False, on_delete=models.CASCADE, related_name='signatures',
        to=DocumentFile, verbose_name=_('Document file')
    )
    # Basic fields
    date_time = models.DateTimeField(
        blank=True, editable=False, null=True, verbose_name=_(
            'Date and time signed'
        )
    )
    key_id = models.CharField(
        help_text=_('ID of the key that will be used to sign the document.'),
        max_length=40, verbose_name=_('Key ID')
    )
    # With proper key
    signature_id = models.CharField(
        blank=True, editable=False, null=True, max_length=64,
        verbose_name=_('Signature ID')
    )
    public_key_fingerprint = models.CharField(
        blank=True, editable=False, null=True, max_length=40,
        verbose_name=_('Public key fingerprint')
    )

    objects = InheritanceManager()

    class Meta:
        ordering = ('pk',)
        verbose_name = _('Document file signature')
        verbose_name_plural = _('Document file signatures')

    def __str__(self):
        return self.signature_id or '{} - {}'.format(
            self.date_time, self.key_id
        )

    def get_absolute_url(self):
        return reverse(
            viewname='signatures:document_file_signature_detail',
            kwargs={'signature_id': self.pk}
        )


class DetachedSignature(ExtraDataModelMixin, SignatureBaseModel):
    signature_file = models.FileField(
        blank=True, help_text=_(
            'Signature file previously generated.'
        ), null=True, storage=DefinedStorageLazy(
            name=STORAGE_NAME_DOCUMENT_SIGNATURES_DETACHED_SIGNATURE
        ), upload_to=upload_to, verbose_name=_('Signature file')
    )

    objects = DetachedSignatureManager()

    class Meta:
        verbose_name = _('Document file detached signature')
        verbose_name_plural = _('Document file detached signatures')

    def __str__(self):
        return '{}-{}'.format(
            self.document_file, _('signature')
        )

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        event=event_detached_signature_deleted,
        target='document_file'
    )
    def delete(self, *args, **kwargs):
        signature_file_name = self.signature_file.name
        if signature_file_name:
            self.signature_file.close()
            self.signature_file.storage.delete(name=signature_file_name)
        super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'self',
            'event': event_detached_signature_uploaded,
            'target': 'document_file'
        }
    )
    def save(self, *args, **kwargs):
        with self.document_file.open() as file_object:
            try:
                verify_result = Key.objects.verify_file(
                    file_object=file_object,
                    signature_file=self.signature_file
                )
            except VerificationError as exception:
                # Not signed
                logger.debug(
                    'detached signature verification error; %s', exception
                )
            else:
                self.signature_file.seek(0)

                # Invalid signatures do not have a date attribute
                self.date_time = getattr(verify_result, 'date_time', None)
                self.key_id = verify_result.key_id
                self.signature_id = verify_result.signature_id
                self.public_key_fingerprint = verify_result.pubkey_fingerprint

        return super().save(*args, **kwargs)


class EmbeddedSignature(SignatureBaseModel):
    objects = EmbeddedSignatureManager()

    class Meta:
        verbose_name = _('Document file embedded signature')
        verbose_name_plural = _('Document file embedded signatures')

    def save(self, *args, **kwargs):
        logger.debug(msg='checking for embedded signature')

        if self.pk:
            raw = True
        else:
            raw = False

        with self.document_file.open(raw=raw) as file_object:
            try:
                verify_result = Key.objects.verify_file(
                    file_object=file_object
                )
            except VerificationError as exception:
                # Not signed
                logger.debug(
                    'embedded signature verification error; %s', exception
                )
            else:
                self.date_time = getattr(verify_result, 'date_time', None)
                self.key_id = verify_result.key_id
                self.signature_id = verify_result.signature_id
                self.public_key_fingerprint = verify_result.pubkey_fingerprint

                # Return must be under the else: context to ensure that an
                # embedded signature instance is created only when valid.
                return super().save(*args, **kwargs)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_document_type_ocr_settings, icon_document_type_ocr_submit,
    icon_document_version_ocr_content_multiple_delete,
    icon_document_version_ocr_content_single_delete,
    icon_document_version_ocr_content_detail,
    icon_document_version_ocr_content_download,
    icon_document_version_ocr_multiple_submit,
    icon_document_version_ocr_single_submit,
    icon_document_version_page_ocr_content_detail,
    icon_document_version_page_ocr_content_edit
)
from .permissions import (
    permission_document_type_ocr_setup, permission_document_version_ocr,
    permission_document_version_ocr_content_edit,
    permission_document_version_ocr_content_view
)

# Document type

link_document_type_ocr_settings = Link(
    args='resolved_object.id',
    icon=icon_document_type_ocr_settings,
    permissions=(permission_document_type_ocr_setup,), text=_('Setup OCR'),
    view='ocr:document_type_ocr_settings'
)
link_document_type_submit = Link(
    icon=icon_document_type_ocr_submit,
    permissions=(permission_document_version_ocr,),
    text=_('OCR documents per type'), view='ocr:document_type_submit'
)

# Document version

link_document_version_ocr_content_multiple_delete = Link(
    icon=icon_document_version_ocr_content_multiple_delete,
    text=_('Delete OCR content'),
    view='ocr:document_version_ocr_content_multiple_delete'
)
link_document_version_ocr_content_single_delete = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_single_delete,
    permissions=(permission_document_version_ocr,),
    text=_('Delete OCR content'),
    view='ocr:document_version_ocr_content_single_delete'
)
link_document_version_ocr_content_detail = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_detail,
    permissions=(permission_document_version_ocr_content_view,),
    text=_('OCR'), view='ocr:document_version_ocr_content_view'
)
link_document_version_ocr_content_download = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_content_download,
    permissions=(permission_document_version_ocr_content_view,),
    text=_('Download OCR text'),
    view='ocr:document_version_ocr_content_download'
)
link_document_version_ocr_multiple_submit = Link(
    icon=icon_document_version_ocr_multiple_submit,
    text=_('Submit for OCR'), view='ocr:document_version_ocr_multiple_submit'
)
link_document_version_ocr_single_submit = Link(
    args='resolved_object.id',
    icon=icon_document_version_ocr_single_submit,
    permissions=(permission_document_version_ocr,), text=_('Submit for OCR'),
    view='ocr:document_version_ocr_single_submit'
)

# Document version page

link_document_version_page_ocr_content_detail_view = Link(
    args='resolved_object.id',
    icon=icon_document_version_page_ocr_content_detail,
    permissions=(permission_document_version_ocr_content_view,),
    text=_('OCR'), view='ocr:document_version_page_ocr_content_detail_view'
)
link_document_version_page_ocr_content_edit_view = Link(
    args='resolved_object.id',
    icon=icon_document_version_page_ocr_content_edit,
    permissions=(permission_document_version_ocr_content_edit,),
    text=_('Edit OCR'),
    view='ocr:document_version_page_ocr_content_edit_view'
)



import logging

import extract_msg

from django.utils.translation import ugettext_lazy as _

from mayan.apps.storage.literals import MSG_MIME_TYPES

from ..classes import FileMetadataDriver

logger = logging.getLogger(__name__)


class ExtractMSGToolDriver(FileMetadataDriver):
    label = _('Extract msg')
    internal_name = 'extract_msg'

    def _process(self, document_file):
        message = extract_msg.Message(
            path=document_file.open()
        )
        result = {}

        if message.cc:
            result['cc'] = message.cc

        if message.sender:
            result['sender'] = message.sender

        if message.subject:
            result['subject'] = message.subject

        if message.to:
            result['to'] = message.to

        logger.debug('message file metadata: %s', result)

        return result


ExtractMSGToolDriver.register(
    mimetypes=list(MSG_MIME_TYPES)
)



from django.apps import apps
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_multi_item, menu_object, menu_secondary, menu_tools
)
from mayan.apps.databases.classes import ModelFieldRelated, ModelProperty
from mayan.apps.documents.signals import signal_post_document_file_upload
from mayan.apps.events.classes import ModelEventType
from mayan.apps.navigation.classes import SourceColumn

from .drivers import *  # NOQA
from .events import (
    event_file_metadata_document_file_finished,
    event_file_metadata_document_file_submitted
)
from .handlers import (
    handler_initialize_new_document_type_file_metadata_settings,
    process_document_file_metadata
)
from .links import (
    link_document_file_metadata_driver_attribute_list,
    link_document_file_metadata_driver_list,
    link_document_file_metadata_single_submit,
    link_document_file_metadata_submit_multiple,
    link_document_type_file_metadata_settings,
    link_document_type_file_metadata_submit
)
from .methods import (
    method_document_file_metadata_submit,
    method_document_file_metadata_submit_single,
    method_get_document_file_file_metadata
)
from .permissions import (
    permission_document_type_file_metadata_setup,
    permission_file_metadata_submit, permission_file_metadata_view
)
from .property_helpers import (
    DocumentFileMetadataHelper, DocumentFileFileMetadataHelper
)


class FileMetadataApp(MayanAppConfig):
    app_namespace = 'file_metadata'
    app_url = 'file_metadata'
    has_tests = True
    name = 'mayan.apps.file_metadata'
    verbose_name = _('File metadata')

    def ready(self):
        super().ready()

        FileMetadataEntry = self.get_model(model_name='FileMetadataEntry')
        DocumentFileDriverEntry = self.get_model(
            model_name='DocumentFileDriverEntry'
        )
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        DocumentFile = apps.get_model(
            app_label='documents', model_name='DocumentFile'
        )
        DocumentTypeSettings = self.get_model(
            model_name='DocumentTypeSettings'
        )
        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )

        Document.add_to_class(
            name='file_metadata_value_of',
            value=DocumentFileMetadataHelper.constructor
        )
        Document.add_to_class(
            name='submit_for_file_metadata_processing',
            value=method_document_file_metadata_submit
        )

        DocumentFile.add_to_class(
            name='file_metadata_value_of',
            value=DocumentFileFileMetadataHelper.constructor
        )
        DocumentFile.add_to_class(
            name='get_file_metadata',
            value=method_get_document_file_file_metadata
        )
        DocumentFile.add_to_class(
            name='submit_for_file_metadata_processing',
            value=method_document_file_metadata_submit_single
        )

        ModelEventType.register(
            model=Document, event_types=(
                event_file_metadata_document_file_finished,
                event_file_metadata_document_file_submitted
            )
        )

        ModelFieldRelated(
            label=_('File metadata key'), model=Document,
            name='files__file_metadata_drivers__entries__key'
        )
        ModelFieldRelated(
            label=_('File metadata value'), model=Document,
            name='files__file_metadata_drivers__entries__value'
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_file_metadata_submit,
                permission_file_metadata_view
            )
        )
        ModelPermission.register(
            model=DocumentType, permissions=(
                permission_document_type_file_metadata_setup,
                permission_file_metadata_submit,
                permission_file_metadata_view
            )
        )
        ModelPermission.register_inheritance(
            model=DocumentTypeSettings, related='document_type'
        )
        ModelPermission.register_inheritance(
            model=DocumentFileDriverEntry, related='document_file'
        )

        ModelProperty(
            description=_(
                'Return the value of a specific file metadata.'
            ), label=_('File metadata value of'), model=Document,
            name='file_metadata_value_of.< underscore separated driver name and property name >'
        )
        ModelProperty(
            description=_(
                'Return the value of a specific file metadata.'
            ), label=_('File metadata value of'), model=DocumentFile,
            name='file_metadata_value_of.< underscore separated driver name and property name >'
        )

        SourceColumn(
            attribute='key', is_identifier=True, source=FileMetadataEntry
        )
        SourceColumn(
            attribute='value', include_label=True, source=FileMetadataEntry
        )
        SourceColumn(
            attribute='driver', is_identifier=True,
            source=DocumentFileDriverEntry
        )
        SourceColumn(
            attribute='driver__internal_name', include_label=True,
            source=DocumentFileDriverEntry
        )
        SourceColumn(
            attribute='get_attribute_count', include_label=True,
            source=DocumentFileDriverEntry
        )

        menu_tools.bind_links(
            links=(link_document_type_file_metadata_submit,)
        )

        # Document file

        menu_list_facet.bind_links(
            links=(link_document_file_metadata_driver_list,),
            sources=(DocumentFile,)
        )

        menu_multi_item.bind_links(
            links=(link_document_file_metadata_submit_multiple,),
            sources=(DocumentFile,)
        )

        menu_secondary.bind_links(
            links=(link_document_file_metadata_single_submit,), sources=(
                'file_metadata:document_file_metadata_driver_list',
                'file_metadata:document_file_metadata_driver_attribute_list'
            )
        )

        # Document file driver

        menu_object.bind_links(
            links=(link_document_file_metadata_driver_attribute_list,),
            sources=(DocumentFileDriverEntry,)
        )

        # Document type

        menu_list_facet.bind_links(
            links=(link_document_type_file_metadata_settings,),
            sources=(DocumentType,)
        )

        post_save.connect(
            dispatch_uid='file_metadata_handler_initialize_new_document_type_file_metadata_settings',
            receiver=handler_initialize_new_document_type_file_metadata_settings,
            sender=DocumentType
        )
        signal_post_document_file_upload.connect(
            dispatch_uid='file_metadata_process_document_file_metadata',
            receiver=process_document_file_metadata,
            sender=DocumentFile
        )



from datetime import timedelta
import logging

from django.apps import apps
from django.contrib.auth import get_user_model
from django.db import models
from django.utils.timezone import now

from mayan.apps.databases.classes import ModelQueryFields

from .settings import (
    setting_favorite_count, setting_recently_accessed_document_count,
    setting_recently_created_document_count
)

logger = logging.getLogger(name=__name__)


class DocumentManager(models.Manager):
    def get_by_natural_key(self, uuid):
        return self.get(
            uuid=str(uuid)
        )


class DocumentFileManager(models.Manager):
    def get_by_natural_key(self, checksum, document_natural_key):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        try:
            document = Document.objects.get_by_natural_key(
                *document_natural_key
            )
        except Document.DoesNotExist:
            raise self.model.DoesNotExist

        return self.get(document__pk=document.pk, checksum=checksum)


class DocumentFilePageManager(models.Manager):
    def get_by_natural_key(self, page_number, document_file_natural_key):
        DocumentFile = apps.get_model(
            app_label='documents', model_name='DocumentFile'
        )
        try:
            document_file = DocumentFile.objects.get_by_natural_key(
                *document_file_natural_key
            )
        except DocumentFile.DoesNotExist:
            raise self.model.DoesNotExist

        return self.get(
            document_file__pk=document_file.pk, page_number=page_number
        )


class DocumentTypeManager(models.Manager):
    def check_delete_periods(self):
        logger.info(msg='Executing')

        for document_type in self.all():
            logger.info(
                'Checking deletion period of document type: %s',
                document_type
            )
            if document_type.delete_time_period and document_type.delete_time_unit:
                delta = timedelta(
                    **{
                        document_type.delete_time_unit: document_type.delete_time_period
                    }
                )
                logger.info(
                    'Document type: %s, has a deletion period delta of: %s',
                    document_type, delta
                )
                for document in document_type.trashed_documents.filter(trashed_date_time__lt=now() - delta):
                    logger.info(
                        'Document "%s" with id: %d, trashed on: %s, exceeded '
                        'delete period', document, document.pk,
                        document.trashed_date_time
                    )
                    try:
                        document.delete()
                    except Exception as exception:
                        logger.error(
                            'Unable to delete trashed document ID: %d; %s',
                            document.pk, exception
                        )
            else:
                logger.info(
                    'Document type: %s, has a no retention delta',
                    document_type
                )

        logger.info(msg='Finished')

    def check_trash_periods(self):
        logger.info(msg='Executing')

        for document_type in self.all():
            logger.info(
                'Checking trash period of document type: %s', document_type
            )
            if document_type.trash_time_period and document_type.trash_time_unit:
                delta = timedelta(
                    **{
                        document_type.trash_time_unit: document_type.trash_time_period
                    }
                )
                logger.info(
                    'Document type: %s, has a trash period delta of: %s',
                    document_type, delta
                )
                for document in document_type.documents.filter(datetime_created__lt=now() - delta):
                    logger.info(
                        'Document "%s" with id: %d, added on: %s, exceeded '
                        'trash period', document, document.pk,
                        document.datetime_created
                    )
                    document.delete()
            else:
                logger.info(
                    'Document type: %s, has a no retention delta',
                    document_type
                )

        logger.info(msg='Finished')

    def document_stubs_delete(self):
        logger.info(msg='Executing')

        for document_type in self.all():
            logger.info(
                'Checking expired document stubs of document type: %s',
                document_type
            )

            if document_type.document_stub_pruning_enabled:
                stale_stub_documents = document_type.documents.filter(
                    is_stub=True, datetime_created__lt=now() - timedelta(
                        seconds=document_type.document_stub_expiration_interval
                    )
                )
                logger.debug(
                    'Deleting %d document stubs', stale_stub_documents.count()
                )
                for stale_stub_document in stale_stub_documents:
                    stale_stub_document.delete(to_trash=False)

        logger.info(msg='Finished')

    def get_by_natural_key(self, label):
        return self.get(label=label)


class FavoriteDocumentManager(models.Manager):
    def get_by_natural_key(
        self, datetime_accessed, document_natural_key, user_natural_key
    ):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        User = get_user_model()
        try:
            document = Document.objects.get_by_natural_key(
                *document_natural_key
            )
        except Document.DoesNotExist:
            raise self.model.DoesNotExist
        else:
            try:
                user = User.objects.get_by_natural_key(*user_natural_key)
            except User.DoesNotExist:
                raise self.model.DoesNotExist

        return self.get(document__pk=document.pk, user__pk=user.pk)


class RecentlyAccessedDocumentManager(models.Manager):
    def get_by_natural_key(
        self, datetime_accessed, document_natural_key, user_natural_key
    ):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        User = get_user_model()
        try:
            document = Document.objects.get_by_natural_key(
                *document_natural_key
            )
        except Document.DoesNotExist:
            raise self.model.DoesNotExist
        else:
            try:
                user = User.objects.get_by_natural_key(*user_natural_key)
            except User.DoesNotExist:
                raise self.model.DoesNotExist

        return self.get(
            document__pk=document.pk, user__pk=user.pk,
            datetime_accessed=datetime_accessed
        )


class TrashCanManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(in_trash=True)


class TrashCanQuerySet(models.QuerySet):
    def delete(self, to_trash=True):
        for instance in self:
            instance.delete(to_trash=to_trash)


class ValidDocumentManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(
            in_trash=False
        )


class ValidDocumentFileManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(document__in_trash=False)


class ValidDocumentFilePageManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(
            document_file__document__in_trash=False
        )


class ValidDocumentVersionManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(document__in_trash=False)


class ValidDocumentVersionPageManager(models.Manager):
    def get_queryset(self):
        return models.QuerySet(
            model=self.model, using=self._db
        ).filter(
            document_version__document__in_trash=False
        )


class ValidFavoriteDocumentManager(models.Manager):
    def add_for_user(self, user, document):
        favorite_document, created = self.model.objects.get_or_create(
            user=user, document=document
        )

        # Delete old (by date) favorites in excess of the values of
        # setting_favorite_count.
        favorites_to_delete = self.filter(user=user).values('pk').order_by(
            '-datetime_added'
        )[setting_favorite_count.value:]
        self.filter(pk__in=favorites_to_delete).delete()

        return favorite_document

    def get_for_user(self, user):
        FavoriteDocumentProxy = apps.get_model(
            app_label='documents', model_name='FavoriteDocumentProxy'
        )

        if user.is_authenticated:
            return FavoriteDocumentProxy.valid.filter(favorites__user=user)
        else:
            return FavoriteDocumentProxy.valid.none()

    def get_queryset(self):
        return super().get_queryset().filter(
            document__in_trash=False
        )

    def remove_for_user(self, user, document):
        self.get(user=user, document=document).delete()


class ValidFavoriteDocumentProxyManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(
            in_trash=False
        )


class ValidRecentlyAccessedDocumentManager(models.Manager):
    def add_document_for_user(self, user, document):
        if user.is_authenticated:
            new_recent, created = self.model.objects.get_or_create(
                user=user, document=document
            )
            if not created:
                # Document already in the recent list, just save to force
                # accessed date and time update.
                new_recent.save(
                    update_fields=('datetime_accessed',)
                )

            recent_to_delete = self.filter(user=user).values_list(
                'pk', flat=True
            )[
                setting_recently_accessed_document_count.value:
            ]

            self.filter(
                pk__in=list(recent_to_delete)
            ).delete()
        return new_recent

    def get_for_user(self, user):
        RecentlyAccessedDocumentProxy = apps.get_model(
            app_label='documents', model_name='RecentlyAccessedDocumentProxy'
        )

        if user.is_authenticated:
            return RecentlyAccessedDocumentProxy.valid.filter(
                recent__user=user
            ).order_by('-recent__datetime_accessed')
        else:
            return RecentlyAccessedDocumentProxy.objects.none()

    def get_queryset(self):
        return super().get_queryset().filter(
            document__in_trash=False
        )


class ValidRecentlyCreatedDocumentManager(models.Manager):
    def get_queryset(self):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        queryset = ModelQueryFields.get(
            model=Document
        ).get_queryset(manager_name='valid')

        return super().get_queryset().filter(
            pk__in=queryset.order_by('-datetime_created')[
                :setting_recently_created_document_count.value
            ].values('pk')
        ).order_by('-datetime_created')



from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import (
    EventManagerMethodAfter, EventManagerSave
)

from .events import event_smart_link_created, event_smart_link_edited
from .literals import INCLUSION_AND, INCLUSION_CHOICES, OPERATOR_CHOICES
from .managers import SmartLinkManager
from .model_mixins import (
    ResolvedSmartLinkBusinessLogicMixin, SmartLinkBusinessLogicMixin,
    SmartLinkConditionBusinessLogicMixin
)


class SmartLink(
    ExtraDataModelMixin, SmartLinkBusinessLogicMixin, models.Model
):
    """
    This model stores the basic fields for a smart link. Smart links allow
    linking documents using a programmatic method of conditions that mirror
    Django's database filter operations.
    """
    label = models.CharField(
        db_index=True, help_text=_('A short text describing the smart link.'),
        max_length=128, verbose_name=_('Label')
    )
    dynamic_label = models.CharField(
        blank=True, max_length=96, help_text=_(
            'Use this field to show a unique label depending on the '
            'document from which the smart link is being accessed.'
        ), verbose_name=_('Dynamic label')
    )
    enabled = models.BooleanField(
        default=True, verbose_name=_('Enabled')
    )
    document_types = models.ManyToManyField(
        related_name='smart_links', to=DocumentType,
        verbose_name=_('Document types')
    )

    objects = SmartLinkManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Smart link')
        verbose_name_plural = _('Smart links')

    def __str__(self):
        return self.label

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_smart_link_created,
            'target': 'self'
        },
        edited={
            'event': event_smart_link_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class ResolvedSmartLink(ResolvedSmartLinkBusinessLogicMixin, SmartLink):
    """
    Proxy model to represent an already resolved smart link. Used for easier
    columns registration.
    """
    class Meta:
        proxy = True


class SmartLinkCondition(
    ExtraDataModelMixin, SmartLinkConditionBusinessLogicMixin, models.Model
):
    """
    This model stores a single smart link condition. A smart link is a
    collection of one of more smart link conditions.
    """
    smart_link = models.ForeignKey(
        on_delete=models.CASCADE, related_name='conditions', to=SmartLink,
        verbose_name=_('Smart link')
    )
    inclusion = models.CharField(
        choices=INCLUSION_CHOICES, default=INCLUSION_AND,
        help_text=_('The inclusion is ignored for the first item.'),
        max_length=16
    )
    foreign_document_data = models.CharField(
        help_text=_('This represents the metadata of all other documents.'),
        max_length=128, verbose_name=_('Foreign document attribute')
    )
    operator = models.CharField(choices=OPERATOR_CHOICES, max_length=16)
    expression = models.TextField(
        help_text=_(
            'The expression using document properties to be evaluated '
            'against the foreign document field.'
        ), verbose_name=_('Expression')
    )
    negated = models.BooleanField(
        default=False, help_text=_('Inverts the logic of the operator.'),
        verbose_name=_('Negated')
    )
    enabled = models.BooleanField(
        default=True, verbose_name=_('Enabled')
    )

    class Meta:
        verbose_name = _('Link condition')
        verbose_name_plural = _('Link conditions')

    def __str__(self):
        return self.get_full_label()

    @method_event(
        event_manager_class=EventManagerMethodAfter,
        event=event_smart_link_edited,
        target='smart_link'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'self',
            'event': event_smart_link_edited,
            'target': 'smart_link'
        },
        edited={
            'action_object': 'self',
            'event': event_smart_link_edited,
            'target': 'smart_link'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0013_auto_20191005_0646')
    ]

    operations = [
        migrations.AlterField(
            model_name='documenttypemetadatatype',
            name='metadata_type',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='document_types', to='metadata.MetadataType',
                verbose_name='Metadata type'
            )
        ),
        migrations.AlterField(
            model_name='metadatatype',
            name='default',
            field=models.CharField(
                blank=True, help_text='Enter a template to render.',
                max_length=128, null=True, verbose_name='Default'
            )
        ),
        migrations.AlterField(
            model_name='metadatatype',
            name='lookup',
            field=models.TextField(
                blank=True, help_text='Enter a template to render. Must '
                'result in a comma delimited string.', null=True,
                verbose_name='Lookup'
            )
        )
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(
    label=_('Document exports'), name='document_exports'
)

permission_document_version_export = namespace.add_permission(
    label=_('Export document versions'),
    name='document_version_export'
)



from pytz import common_timezones

from django.conf import settings
from django.utils import translation


def get_language_choices():
    return sorted(
        settings.LANGUAGES, key=lambda entry: entry[1]
    )


def to_language(promise, language):
    current_language = translation.get_language()
    translation.activate(language=language)
    result = str(promise)
    translation.activate(language=current_language)

    return result


def get_timezone_choices():
    return zip(common_timezones, common_timezones)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(
    label=_('Indexing'), name='document_indexing'
)

permission_index_instance_view = namespace.add_permission(
    label=_('View index instances'), name='document_index_instance_view'
)
permission_index_template_create = namespace.add_permission(
    label=_('Create new index templates'), name='document_index_create'
)
permission_index_template_edit = namespace.add_permission(
    label=_('Edit index templates'), name='document_index_edit'
)
permission_index_template_delete = namespace.add_permission(
    label=_('Delete index templates'), name='document_index_delete'
)
permission_index_template_view = namespace.add_permission(
    label=_('View index templates'), name='document_index_view'
)
permission_index_template_rebuild = namespace.add_permission(
    label=_('Rebuild index templates'), name='document_rebuild_indexes'
)



from ..classes import CredentialBackend


class TestCredentialBackend(CredentialBackend):
    label = 'Test source backend'

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields.update(
            {
                'test_field': {
                    'label': 'Test field',
                    'class': 'django.forms.CharField',
                    'required': False
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                'Testing', {
                    'fields': ('test_field')
                },
            ),
        )

        return fieldsets

    def process_documents(self, dry_run=False):
        """Do nothing. This method is added to allow view testing."""


class TestCredentialBackendWithPostProcessing(TestCredentialBackend):
    label = 'Test source backend with post processing'

    @classmethod
    def post_processing(cls, obj):
        return obj



from mayan.apps.testing.tests.base import BaseTestCase

from ..events import event_otp_disabled, event_otp_enabled

from .mixins import AuthenticationOTPTestMixin


class UserOTPDataTestCase(AuthenticationOTPTestMixin, BaseTestCase):
    create_test_case_super_user = True

    def test_method_get_absolute_url(self):
        self._test_case_super_user.otp_data.get_absolute_url()

    def test_otp_disable(self):
        self._enable_test_otp()

        self._clear_events()

        self._test_case_super_user.otp_data.disable()

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_otp_disabled.id)

    def test_otp_enable(self):
        self._clear_events()

        self._enable_test_otp()

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_super_user)
        self.assertEqual(events[0].target, self._test_case_super_user)
        self.assertEqual(events[0].verb, event_otp_enabled.id)



from ...tasks import task_index_instances, task_reindex_backend

from .base import SearchTestMixin


class SearchTaskTestMixin(SearchTestMixin):
    def _execute_task_reindex_backend(self):
        task_reindex_backend.apply_async().get()

    def _execute_task_index_instances(self):
        task_index_instances.apply_async(
            kwargs={
                'id_list': (self._test_object.pk,),
                'search_model_full_name': self._test_model_search.full_name
            }
        ).get()



import os

from django.conf import settings

DOCUMENT_SIGNATURE_DETACHED_ACTION_CLASS_PATH = 'mayan.apps.document_signatures.workflow_actions.DocumentSignatureDetachedAction'
DOCUMENT_SIGNATURE_EMBEDDED_ACTION_CLASS_PATH = 'mayan.apps.document_signatures.workflow_actions.DocumentSignatureEmbeddedAction'
TEST_SIGNED_DOCUMENT_PATH = os.path.join(
    settings.BASE_DIR, 'apps', 'document_signatures', 'tests', 'contrib',
    'sample_documents', 'mayan_11_1.pdf.gpg'
)
TEST_SIGNATURE_FILE_PATH = os.path.join(
    settings.BASE_DIR, 'apps', 'document_signatures', 'tests', 'contrib',
    'sample_documents', 'mayan_11_1.pdf.sig'
)
TEST_SIGNATURE_ID = 'XVkoGKw35yU1iq11dZPiv7uAY7k'

TEST_SIGNED_DOCUMENT_COUNT = 2
TEST_UNSIGNED_DOCUMENT_COUNT = 4



from django.db import models
from django.utils.translation import ugettext_lazy as _

from .literals import DEFAULT_LOCK_MANAGER_DEFAULT_LOCK_TIMEOUT
from .managers import LockManager
from .model_mixins import LockBusinessLogicMixin
from .settings import setting_default_lock_timeout


class Lock(LockBusinessLogicMixin, models.Model):
    """
    Model to provide distributed resource locking using the database.
    """
    creation_datetime = models.DateTimeField(
        auto_now_add=True, verbose_name=_('Creation datetime')
    )
    timeout = models.IntegerField(
        default=DEFAULT_LOCK_MANAGER_DEFAULT_LOCK_TIMEOUT,
        verbose_name=_('Timeout')
    )
    name = models.CharField(
        max_length=255, unique=True, verbose_name=_('Name')
    )

    objects = LockManager()

    class Meta:
        verbose_name = _('Lock')
        verbose_name_plural = _('Locks')

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.timeout and not kwargs.get('timeout'):
            self.timeout = setting_default_lock_timeout.value

        super().save(*args, **kwargs)



import logging

from django.core.exceptions import ValidationError
from django.db.utils import IntegrityError
from django.utils.translation import ugettext_lazy as _

from mayan.apps.document_states.classes import WorkflowAction
from mayan.apps.document_states.exceptions import WorkflowStateActionError
from mayan.apps.document_states.models.workflow_instance_models import WorkflowInstance

from .models import DocumentMetadata, MetadataType
from .permissions import (
    permission_document_metadata_add, permission_document_metadata_edit,
    permission_document_metadata_remove
)

logger = logging.getLogger(name=__name__)


class DocumentMetadataAddAction(WorkflowAction):
    form_field_widgets = {
        'metadata_types': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        }
    }
    label = _('Add metadata')
    permission = permission_document_metadata_add

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        document_types_queryset = cls.workflow_template_state.workflow.document_types

        metadata_type_queryset = MetadataType.objects.get_for_document_types(
            queryset=document_types_queryset
        )

        fields.update(
            {
                'metadata_types': {
                    'class': 'mayan.apps.views.fields.FormFieldFilteredModelChoiceMultiple',
                    'help_text': _(
                        'Metadata types to add to the document.'
                    ),
                    'kwargs': {
                        'source_queryset': metadata_type_queryset,
                        'permission': cls.permission
                    },
                    'label': _('Metadata types'),
                    'required': True
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Metadata types'), {
                    'fields': ('metadata_types',)
                },
            ),
        )
        return fieldsets

    def execute(self, context):
        for metadata_type in self.get_metadata_types():
            try:
                context['workflow_instance'].document.metadata.create(metadata_type=metadata_type)
            except IntegrityError as exception:
                """This document already has the metadata type added."""
                raise WorkflowStateActionError(
                    _(
                        'Unable to add metadata type "%(metadata_type)s" '
                        'from document: %(document)s. Exception: '
                        '%(exception)s'
                    ) % {
                        'document': context['workflow_instance'].document,
                        'exception': exception,
                        'metadata_type': metadata_type
                    }
                ) from exception

    def get_metadata_types(self):
        return MetadataType.objects.filter(
            pk__in=self.kwargs.get(
                'metadata_types', ()
            )
        )


class DocumentMetadataEditAction(WorkflowAction):
    form_field_widgets = {
        'metadata_type': {
            'class': 'django.forms.widgets.Select', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        }
    }
    form_fields = {
        'value': {
            'label': _('Value'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    'Value to assign to the metadata. '
                    'Can be a literal value or template code.'
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': True
            }
        }
    }
    label = _('Edit metadata')

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        document_types_queryset = cls.workflow_template_state.workflow.document_types

        metadata_type_queryset = MetadataType.objects.get_for_document_types(
            queryset=document_types_queryset
        )

        fields.update(
            {
                'metadata_type': {
                    'class': 'mayan.apps.views.fields.FormFieldFilteredModelChoice',
                    'help_text': _(
                        'Metadata types to edit.'
                    ),
                    'kwargs': {
                        'source_queryset': metadata_type_queryset,
                        'permission': permission_document_metadata_edit
                    },
                    'label': _('Metadata type'),
                    'required': True
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Metadata'), {
                    'fields': ('metadata_type', 'value')
                },
            ),
        )
        return fieldsets

    def execute(self, context):
        try:
            metadata_type = self.get_metadata_type()
            document_metadata = context['workflow_instance'].document.metadata.get(
                metadata_type=metadata_type
            )
        except DocumentMetadata.DoesNotExist as exception:
            """
            Non fatal, we just ignore the action to edit the metadata value.
            """
            raise WorkflowStateActionError(
                _(
                    'Unable to edit metadata type "%(metadata_type)s" '
                    'from document: %(document)s. Document does not have '
                    'the metadata type to be edited. Exception: '
                    '%(exception)s'
                ) % {
                    'document': context['workflow_instance'].document,
                    'exception': exception,
                    'metadata_type': metadata_type
                }
            ) from exception
        else:
            document_metadata.value = self.render_field(
                field_name='value', context=context
            )
            document_metadata.save()

    def get_metadata_type(self):
        return MetadataType.objects.get(
            pk=self.kwargs['metadata_type']
        )


class DocumentMetadataRemoveAction(DocumentMetadataAddAction):
    label = _('Remove metadata')
    permission = permission_document_metadata_remove

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields['metadata_types']['help_text'] = _(
            'Metadata types to remove from the document.'
        )

        return fields

    def execute(self, context):
        for metadata_type in self.get_metadata_types():
            try:
                context['workflow_instance'].document.metadata.get(
                    metadata_type=metadata_type
                ).delete()
            except DocumentMetadata.DoesNotExist:
                """This document does not have the metadata type added."""
            except ValidationError as exception:
                raise WorkflowStateActionError(
                    _(
                        'Unable to remove metadata type "%(metadata_type)s" '
                        'from document: %(document)s. Exception: '
                        '%(exception)s'
                    ) % {
                        'document': context['workflow_instance'].document,
                        'exception': exception,
                        'metadata_type': metadata_type
                    }
                ) from exception



from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Quota',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'backend_path', models.CharField(
                        help_text='The dotted Python path to the backend '
                        'class.', max_length=255, verbose_name='Backend path'
                    )
                ),
                (
                    'backend_data', models.TextField(
                        blank=True, verbose_name='Backend data'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, help_text='Allow quick disable or '
                        'enable of the quota.', verbose_name='Enabled'
                    )
                )
            ],
            options={
                'verbose_name': 'Quota',
                'verbose_name_plural': 'Quotas'
            }
        )
    ]



from django.conf.urls import url

from .api_views.document_api_views import (
    APIDocumentChangeTypeView, APIDocumentDetailView,
    APIDocumentFileActionListView, APIDocumentListView, APIDocumentUploadView
)
from .api_views.document_file_api_views import (
    APIDocumentFileDetailView, APIDocumentFileListView,
    APIDocumentFilePageDetailView, APIDocumentFilePageImageView,
    APIDocumentFilePageListView
)
from .api_views.document_type_api_views import (
    APIDocumentTypeDetailView, APIDocumentTypeListView,
    APIDocumentTypeQuickLabelDetailView, APIDocumentTypeQuickLabelListView
)
from .api_views.document_version_api_views import (
    APIDocumentVersionDetailView, APIDocumentVersionListView,
    APIDocumentVersionModificationBackendListView,
    APIDocumentVersionModificationView, APIDocumentVersionPageDetailView,
    APIDocumentVersionPageImageView, APIDocumentVersionPageListView
)
from .api_views.favorite_document_api_views import (
    APIFavoriteDocumentDetailView, APIFavoriteDocumentListView
)
from .api_views.recently_accessed_document_api_views import (
    APIRecentlyAccessedDocumentListView
)
from .api_views.recently_created_document_api_views import (
    APIRecentlyCreatedDocumentListView
)
from .api_views.trashed_document_api_views import (
    APITrashedDocumentDetailView, APITrashedDocumentImageView,
    APITrashedDocumentListView, APITrashedDocumentRestoreView
)
from .views.document_file_page_views import (
    DocumentFilePageListView, DocumentFilePageNavigationFirst,
    DocumentFilePageNavigationLast, DocumentFilePageNavigationNext,
    DocumentFilePageNavigationPrevious, DocumentFilePageRotateLeftView,
    DocumentFilePageRotateRightView, DocumentFilePageView,
    DocumentFilePageViewResetView, DocumentFilePageZoomInView,
    DocumentFilePageZoomOutView
)
from .views.document_file_views import (
    DocumentFileDeleteView, DocumentFileEditView, DocumentFileIntrospectView,
    DocumentFileListView, DocumentFilePreviewView, DocumentFilePrintFormView,
    DocumentFilePrintView, DocumentFilePropertiesView,
    DocumentFileTransformationsClearView,
    DocumentFileTransformationsCloneView
)
from .views.document_type_retention_policy_views import DocumentTypeRetentionPoliciesEditView
from .views.document_type_views import (
    DocumentTypeCreateView, DocumentTypeDeleteView,
    DocumentTypeDocumentListView, DocumentTypeEditView,
    DocumentTypeFilenameCreateView, DocumentTypeFilenameDeleteView,
    DocumentTypeFilenameEditView, DocumentTypeFilenameGeneratorEditView,
    DocumentTypeFilenameListView, DocumentTypeListView
)
from .views.document_version_page_views import (
    DocumentVersionPageDeleteView, DocumentVersionPageListRemapView,
    DocumentVersionPageListView, DocumentVersionPageNavigationFirst,
    DocumentVersionPageNavigationLast, DocumentVersionPageNavigationNext,
    DocumentVersionPageNavigationPrevious, DocumentVersionPageRotateLeftView,
    DocumentVersionPageRotateRightView, DocumentVersionPageView,
    DocumentVersionPageViewResetView, DocumentVersionPageZoomInView,
    DocumentVersionPageZoomOutView
)
from .views.document_version_views import (
    DocumentVersionActiveView, DocumentVersionCreateView,
    DocumentVersionDeleteView, DocumentVersionEditView,
    DocumentVersionListView, DocumentVersionModifyView,
    DocumentVersionPreviewView, DocumentVersionPrintFormView,
    DocumentVersionPrintView, DocumentVersionTransformationsClearView,
    DocumentVersionTransformationsCloneView
)
from .views.document_views import (
    DocumentListView, DocumentPreviewView, DocumentPropertiesEditView,
    DocumentPropertiesView, DocumentTypeChangeView
)
from .views.favorite_document_views import (
    FavoriteAddView, FavoriteDocumentListView, FavoriteRemoveView
)
from .views.recently_accessed_document_views import (
    RecentlyAccessedDocumentListView
)
from .views.recently_created_document_views import (
    RecentCreatedDocumentListView
)
from .views.trashed_document_views import (
    DocumentTrashView, EmptyTrashCanView, TrashedDocumentDeleteView,
    TrashedDocumentListView, TrashedDocumentRestoreView
)

urlpatterns_document_files = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/files/$',
        name='document_file_list', view=DocumentFileListView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/preview/$',
        name='document_file_preview', view=DocumentFilePreviewView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/delete/$',
        name='document_file_delete',
        view=DocumentFileDeleteView.as_view()
    ),
    url(
        regex=r'^documents/files/multiple/delete/$',
        name='document_file_multiple_delete',
        view=DocumentFileDeleteView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/edit/$',
        name='document_file_edit',
        view=DocumentFileEditView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/introspect/$',
        name='document_file_introspect_single',
        view=DocumentFileIntrospectView.as_view()
    ),
    url(
        regex=r'^documents/files/multiple/introspect/$',
        name='document_file_introspect_multiple',
        view=DocumentFileIntrospectView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/print/form/$',
        name='document_file_print_form',
        view=DocumentFilePrintFormView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/print/$',
        name='document_file_print_view', view=DocumentFilePrintView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/properties/$',
        name='document_file_properties',
        view=DocumentFilePropertiesView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/transformations/clear/$',
        name='document_file_transformations_clear',
        view=DocumentFileTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/files/multiple/transformations/clear/$',
        name='document_file_multiple_transformations_clear',
        view=DocumentFileTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/transformations/clone/$',
        name='document_file_transformations_clone',
        view=DocumentFileTransformationsCloneView.as_view()
    )
]

urlpatterns_document_file_pages = [
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/pages/$',
        name='document_file_page_list', view=DocumentFilePageListView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/$',
        name='document_file_page_view', view=DocumentFilePageView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/navigation/next/$',
        name='document_file_page_navigation_next',
        view=DocumentFilePageNavigationNext.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/navigation/previous/$',
        name='document_file_page_navigation_previous',
        view=DocumentFilePageNavigationPrevious.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/navigation/first/$',
        name='document_file_page_navigation_first',
        view=DocumentFilePageNavigationFirst.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/navigation/last/$',
        name='document_file_page_navigation_last',
        view=DocumentFilePageNavigationLast.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/zoom/in/$',
        name='document_file_page_zoom_in', view=DocumentFilePageZoomInView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/zoom/out/$',
        name='document_file_page_zoom_out', view=DocumentFilePageZoomOutView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/rotate/left/$',
        name='document_file_page_rotate_left',
        view=DocumentFilePageRotateLeftView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/rotate/right/$',
        name='document_file_page_rotate_right',
        view=DocumentFilePageRotateRightView.as_view()
    ),
    url(
        regex=r'^documents/files/pages/(?P<document_file_page_id>\d+)/reset/$',
        name='document_file_page_view_reset',
        view=DocumentFilePageViewResetView.as_view()
    )
]

urlpatterns_document_types = [
    url(
        regex=r'^document_types/$', name='document_type_list',
        view=DocumentTypeListView.as_view()
    ),
    url(
        regex=r'^document_types/create/$', name='document_type_create',
        view=DocumentTypeCreateView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/delete/$',
        name='document_type_delete', view=DocumentTypeDeleteView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/retention_policies/$',
        name='document_type_retention_policies',
        view=DocumentTypeRetentionPoliciesEditView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/documents/$',
        name='document_type_document_list',
        view=DocumentTypeDocumentListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/edit/$',
        name='document_type_edit', view=DocumentTypeEditView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/filename_generator/$',
        name='document_type_filename_generator',
        view=DocumentTypeFilenameGeneratorEditView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/filenames/$',
        name='document_type_filename_list',
        view=DocumentTypeFilenameListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/filenames/create/$',
        name='document_type_filename_create',
        view=DocumentTypeFilenameCreateView.as_view()
    ),
    url(
        regex=r'^document_types/filenames/(?P<document_type_filename_id>\d+)/delete/$',
        name='document_type_filename_delete',
        view=DocumentTypeFilenameDeleteView.as_view()
    ),
    url(
        regex=r'^document_types/filenames/(?P<document_type_filename_id>\d+)/edit/$',
        name='document_type_filename_edit',
        view=DocumentTypeFilenameEditView.as_view()
    )
]

urlpatterns_document_version = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/versions/$',
        name='document_version_list', view=DocumentVersionListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/versions/create/$',
        name='document_version_create',
        view=DocumentVersionCreateView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/active/$',
        name='document_version_active',
        view=DocumentVersionActiveView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/delete/$',
        name='document_version_single_delete',
        view=DocumentVersionDeleteView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/edit/$',
        name='document_version_edit',
        view=DocumentVersionEditView.as_view()
    ),
    url(
        regex=r'^documents/versions/multiple/delete/$',
        name='document_version_multiple_delete',
        view=DocumentVersionDeleteView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/modify/$',
        name='document_version_modify',
        view=DocumentVersionModifyView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/preview/$',
        name='document_version_preview',
        view=DocumentVersionPreviewView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/print/form/$',
        name='document_version_print_form',
        view=DocumentVersionPrintFormView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/print/$',
        name='document_version_print_view',
        view=DocumentVersionPrintView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/transformations/clear/$',
        name='document_version_transformations_clear',
        view=DocumentVersionTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/versions/multiple/transformations/clear/$',
        name='document_version_multiple_transformations_clear',
        view=DocumentVersionTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/transformations/clone/$',
        name='document_version_transformations_clone',
        view=DocumentVersionTransformationsCloneView.as_view()
    )
]

urlpatterns_document_version_pages = [
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/pages/$',
        name='document_version_page_list',
        view=DocumentVersionPageListView.as_view()
    ),
    url(
        regex=r'^documents/versions/pages/(?P<document_version_page_id>\d+)/delete/$',
        name='document_version_page_delete',
        view=DocumentVersionPageDeleteView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/pages/remap/$',
        name='document_version_page_list_remap',
        view=DocumentVersionPageListRemapView.as_view()
    ),
    url(
        regex=r'^documents/versions/pages/(?P<document_version_page_id>\d+)/$',
        name='document_version_page_view',
        view=DocumentVersionPageView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/navigation/next/$',
        name='document_version_page_navigation_next',
        view=DocumentVersionPageNavigationNext.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/navigation/previous/$',
        name='document_version_page_navigation_previous',
        view=DocumentVersionPageNavigationPrevious.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/navigation/first/$',
        name='document_version_page_navigation_first',
        view=DocumentVersionPageNavigationFirst.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/navigation/last/$',
        name='document_version_page_navigation_last',
        view=DocumentVersionPageNavigationLast.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/zoom/in/$',
        name='document_version_page_zoom_in', view=DocumentVersionPageZoomInView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/zoom/out/$',
        name='document_version_page_zoom_out', view=DocumentVersionPageZoomOutView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/rotate/left/$',
        name='document_version_page_rotate_left',
        view=DocumentVersionPageRotateLeftView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/rotate/right/$',
        name='document_version_page_rotate_right',
        view=DocumentVersionPageRotateRightView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_page_id>\d+)/reset/$',
        name='document_version_page_view_reset',
        view=DocumentVersionPageViewResetView.as_view()
    )
]

urlpatterns_documents = [
    url(
        regex=r'^documents/$', name='document_list',
        view=DocumentListView.as_view()
    ),
    url(
        regex=r'^documents/recently/accessed/$',
        name='document_recently_accessed_list',
        view=RecentlyAccessedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/recently/created/$',
        name='document_recently_created_list',
        view=RecentCreatedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/preview/$',
        name='document_preview', view=DocumentPreviewView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/properties/$',
        name='document_properties', view=DocumentPropertiesView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/properties/edit/$',
        name='document_properties_edit',
        view=DocumentPropertiesEditView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/type/$',
        name='document_type_change',
        view=DocumentTypeChangeView.as_view()
    ),
    url(
        regex=r'^documents/multiple/type/$',
        name='document_multiple_type_change',
        view=DocumentTypeChangeView.as_view()
    )
]

urlpatterns_favorite_documents = [
    url(
        regex=r'^documents/favorites/$', name='document_favorite_list',
        view=FavoriteDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/add_to_favorites/$',
        name='document_favorite_add', view=FavoriteAddView.as_view()
    ),
    url(
        regex=r'^documents/multiple/add_to_favorites/$',
        name='document_favorite_add_multiple', view=FavoriteAddView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/remove_from_favorites/$',
        name='document_favorite_remove', view=FavoriteRemoveView.as_view()
    ),
    url(
        regex=r'^documents/multiple/remove_from_favorites/$',
        name='document_favorite_remove_multiple',
        view=FavoriteRemoveView.as_view()
    )
]

urlpatterns_trashed_documents = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/trash/$',
        name='document_trash', view=DocumentTrashView.as_view()
    ),
    url(
        regex=r'^documents/multiple/trash/$', name='document_multiple_trash',
        view=DocumentTrashView.as_view()
    ),
    url(
        regex=r'^trashed_documents/$', name='document_list_deleted',
        view=TrashedDocumentListView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>\d+)/restore/$',
        name='document_restore', view=TrashedDocumentRestoreView.as_view()
    ),
    url(
        regex=r'^trashed_documents/multiple/restore/$',
        name='document_multiple_restore',
        view=TrashedDocumentRestoreView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>\d+)/delete/$',
        name='document_delete', view=TrashedDocumentDeleteView.as_view()
    ),
    url(
        regex=r'^trashed_documents/multiple/delete/$',
        name='document_multiple_delete',
        view=TrashedDocumentDeleteView.as_view()
    ),
    url(
        regex=r'^trash_can/empty/$', name='trash_can_empty',
        view=EmptyTrashCanView.as_view()
    )
]

urlpatterns = []
urlpatterns.extend(urlpatterns_document_files)
urlpatterns.extend(urlpatterns_document_file_pages)
urlpatterns.extend(urlpatterns_document_types)
urlpatterns.extend(urlpatterns_document_version_pages)
urlpatterns.extend(urlpatterns_document_version)
urlpatterns.extend(urlpatterns_documents)
urlpatterns.extend(urlpatterns_favorite_documents)
urlpatterns.extend(urlpatterns_trashed_documents)

api_urls_documents = [
    url(
        regex=r'^document_file_actions/$', name='document-file-action-list',
        view=APIDocumentFileActionListView.as_view()
    ),
    url(
        regex=r'^documents/$', name='document-list',
        view=APIDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/upload/$', name='document-upload',
        view=APIDocumentUploadView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/$',
        name='document-detail',
        view=APIDocumentDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/type/change/$',
        name='document-change-type', view=APIDocumentChangeTypeView.as_view()
    ),
    url(
        regex=r'^documents/accessed/$',
        name='recentlyaccesseddocument-list',
        view=APIRecentlyAccessedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/created/$',
        name='recentlycreateddocument-list',
        view=APIRecentlyCreatedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/favorites/$',
        name='favoritedocument-list',
        view=APIFavoriteDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/favorites/(?P<favorite_document_id>[0-9]+)/$',
        name='favoritedocument-detail',
        view=APIFavoriteDocumentDetailView.as_view()
    )
]

api_urls_document_files = [
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/files/$',
        name='documentfile-list',
        view=APIDocumentFileListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/files/(?P<document_file_id>[0-9]+)/$',
        name='documentfile-detail', view=APIDocumentFileDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/files/(?P<document_file_id>[0-9]+)/pages/$',
        name='documentfilepage-list',
        view=APIDocumentFilePageListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/files/(?P<document_file_id>[0-9]+)/pages/(?P<document_file_page_id>[0-9]+)/$',
        name='documentfilepage-detail',
        view=APIDocumentFilePageDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/files/(?P<document_file_id>[0-9]+)/pages/(?P<document_file_page_id>[0-9]+)/image/$',
        name='documentfilepage-image',
        view=APIDocumentFilePageImageView.as_view()
    )
]

api_urls_document_types = [
    url(
        regex=r'^document_types/$', name='documenttype-list',
        view=APIDocumentTypeListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>[0-9]+)/$',
        name='documenttype-detail', view=APIDocumentTypeDetailView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>[0-9]+)/quick_labels/$',
        name='documenttype-quicklabel-list',
        view=APIDocumentTypeQuickLabelListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>[0-9]+)/quick_labels/(?P<document_type_quick_label_id>[0-9]+)/$',
        name='documenttype-quicklabel-detail',
        view=APIDocumentTypeQuickLabelDetailView.as_view()
    )
]

api_urls_document_versions = [
    url(
        regex=r'^document_version_modification_backends/$',
        name='documentversionmodificationbackend-list',
        view=APIDocumentVersionModificationBackendListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/$',
        name='documentversion-list',
        view=APIDocumentVersionListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/$',
        name='documentversion-detail',
        view=APIDocumentVersionDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/modify/$',
        name='documentversion-modify',
        view=APIDocumentVersionModificationView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/pages/$',
        name='documentversionpage-list',
        view=APIDocumentVersionPageListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/pages/(?P<document_version_page_id>[0-9]+)/$',
        name='documentversionpage-detail',
        view=APIDocumentVersionPageDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/pages/(?P<document_version_page_id>[0-9]+)/image/$',
        name='documentversionpage-image',
        view=APIDocumentVersionPageImageView.as_view()
    )
]

api_urls_trashed_documents = [
    url(
        regex=r'^trashed_documents/$', name='trasheddocument-list',
        view=APITrashedDocumentListView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>[0-9]+)/$',
        name='trasheddocument-detail', view=APITrashedDocumentDetailView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>[0-9]+)/restore/$',
        name='trasheddocument-restore',
        view=APITrashedDocumentRestoreView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>[0-9]+)/image/$',
        name='trasheddocument-image',
        view=APITrashedDocumentImageView.as_view()
    )
]

api_urls = []
api_urls.extend(api_urls_documents)
api_urls.extend(api_urls_document_files)
api_urls.extend(api_urls_document_versions)
api_urls.extend(api_urls_document_types)
api_urls.extend(api_urls_trashed_documents)



from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin

from ..models import StoredDriver

from .literals import (
    TEST_DOCUMENT_FILE_METADATA_KEY, TEST_DOCUMENT_FILE_METADATA_VALUE,
    TEST_STORED_DRIVER_INTERNAL_NAME, TEST_STORED_DRIVER_PATH
)


class DocumentTypeViewTestMixin:
    def _request_document_type_file_metadata_settings_view(self):
        return self.get(
            viewname='file_metadata:document_type_file_metadata_settings',
            kwargs={'document_type_id': self._test_document.document_type.pk}
        )

    def _request_document_type_file_metadata_submit_view(self):
        return self.post(
            viewname='file_metadata:document_type_file_metadata_submit', data={
                'document_type': self._test_document_type.pk
            }
        )


class FileMetadataViewTestMixin:
    def _request_document_file_metadata_driver_list_view(self):
        return self.get(
            viewname='file_metadata:document_file_metadata_driver_list',
            kwargs={'document_file_id': self._test_document_file.pk}
        )

    def _request_document_file_metadata_list_view(self):
        return self.get(
            viewname='file_metadata:document_file_metadata_driver_attribute_list',
            kwargs={
                'document_file_driver_id': self.test_driver.pk
            }
        )

    def _request_document_file_metadata_single_submit_view(self):
        return self.post(
            viewname='file_metadata:document_file_metadata_single_submit',
            kwargs={'document_file_id': self._test_document_file.pk}
        )

    def _request_document_file_multiple_submit_view(self):
        return self.post(
            viewname='file_metadata:document_file_metadata_multiple_submit',
            data={
                'id_list': self._test_document_file.pk
            }
        )


class StoredDriverTestMixin:
    _test_stored_driver_create_auto = True

    def setUp(self):
        super().setUp()

        if self._test_stored_driver_create_auto:
            self._test_stored_driver_create()

    def _test_stored_driver_create(self):
        self._test_stored_driver = StoredDriver.objects.create(
            driver_path=TEST_STORED_DRIVER_PATH,
            internal_name=TEST_STORED_DRIVER_INTERNAL_NAME
        )


class FileMetadataTestMixin(DocumentTestMixin, StoredDriverTestMixin):
    _test_document_file_metadata_create_auto = True

    def setUp(self):
        super().setUp()

        if self._test_document_file_metadata_create_auto:
            self._test_document_file_metadata_create()

    def _test_document_file_metadata_create(self):
        self._test_document_file_driver_entry, created = self._test_document_file.file_metadata_drivers.get_or_create(
            driver=self._test_stored_driver
        )

        self._test_document_file_metadata = self._test_document_file_driver_entry.entries.create(
            key=TEST_DOCUMENT_FILE_METADATA_KEY,
            value=TEST_DOCUMENT_FILE_METADATA_VALUE
        )

        self._test_document_file_metadata_path = '{}__{}'.format(
            self._test_document_file_driver_entry.driver.internal_name,
            self._test_document_file_metadata.key
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import factory_condition_queryset_access

from .icons import (
    icon_announcement_create, icon_announcement_delete,
    icon_announcement_edit, icon_announcement_list
)
from .permissions import (
    permission_announcement_create, permission_announcement_delete,
    permission_announcement_edit, permission_announcement_view
)

link_announcement_create = Link(
    icon=icon_announcement_create, permissions=(
        permission_announcement_create,
    ), text=_('Create announcement'),
    view='announcements:announcement_create'
)
link_announcement_multiple_delete = Link(
    icon=icon_announcement_delete, tags='dangerous', text=_('Delete'),
    view='announcements:announcement_multiple_delete'
)
link_announcement_single_delete = Link(
    args='object.pk', icon=icon_announcement_delete,
    permissions=(permission_announcement_delete,),
    tags='dangerous', text=_('Delete'),
    view='announcements:announcement_single_delete'
)
link_announcement_edit = Link(
    args='object.pk', icon=icon_announcement_edit,
    permissions=(permission_announcement_edit,), text=_('Edit'),
    view='announcements:announcement_edit'
)
link_announcement_list = Link(
    icon=icon_announcement_list, text=_('Announcements'),
    view='announcements:announcement_list'
)
link_announcement_setup = Link(
    condition=factory_condition_queryset_access(
        app_label='announcements', model_name='Announcement',
        object_permission=permission_announcement_view,
        view_permission=permission_announcement_create,
    ), icon=icon_announcement_list,
    text=_('Announcements'), view='announcements:announcement_list'
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from ..icons import (
    icon_document_list, icon_document_preview,
    icon_document_properties_detail, icon_document_properties_edit,
    icon_document_recently_accessed_list,
    icon_document_recently_created_list, icon_document_type_change
)
from ..permissions import (
    permission_document_properties_edit, permission_document_view
)

link_document_type_change = Link(
    args='resolved_object.id', icon=icon_document_type_change,
    permissions=(permission_document_properties_edit,), text=_('Change type'),
    view='documents:document_type_change'
)
link_document_list = Link(
    icon=icon_document_list,
    text=_('All documents'), view='documents:document_list'
)
link_document_recently_accessed_list = Link(
    icon=icon_document_recently_accessed_list, text=_('Recently accessed'),
    view='documents:document_recently_accessed_list'
)
link_document_recently_created_list = Link(
    icon=icon_document_recently_created_list, text=_('Recently created'),
    view='documents:document_recently_created_list'
)
link_document_multiple_type_change = Link(
    text=_('Change type'), icon=icon_document_type_change,
    view='documents:document_multiple_type_change'
)
link_document_preview = Link(
    args='resolved_object.id', icon=icon_document_preview,
    permissions=(permission_document_view,), text=_('Preview'),
    view='documents:document_preview'
)
link_document_properties = Link(
    args='resolved_object.id', icon=icon_document_properties_detail,
    permissions=(permission_document_view,), text=_('Properties'),
    view='documents:document_properties'
)
link_document_properties_edit = Link(
    args='resolved_object.id',
    icon=icon_document_properties_edit,
    permissions=(permission_document_properties_edit,),
    text=_('Edit properties'), view='documents:document_properties_edit'
)



from django.conf.urls import url

from .views import (
    JavaScriptCatalogPublic, UserLocaleProfileDetailView,
    UserLocaleProfileEditView
)

urlpatterns = [
    url(
        regex=r'^jsi18n/$', name='javascript_catalog',
        view=JavaScriptCatalogPublic.as_view()
    ),
    url(
        regex=r'^jsi18n/(?P<packages>\S+?)/$', name='javascript_catalog',
        view=JavaScriptCatalogPublic.as_view()
    ),
    url(
        regex=r'^user/(?P<user_id>\d+)/locale/$',
        name='user_locale_profile_detail',
        view=UserLocaleProfileDetailView.as_view()
    ),
    url(
        regex=r'^user/(?P<user_id>\d+)/locale/edit/$',
        name='user_locale_profile_edit',
        view=UserLocaleProfileEditView.as_view()
    )
]



import logging

from django.apps import apps
from django.contrib.auth import get_user_model

from mayan.apps.lock_manager.backends.base import LockingBackend
from mayan.apps.lock_manager.exceptions import LockError
from mayan.celery import app

from .classes import FileMetadataDriver
from .literals import LOCK_EXPIRE

logger = logging.getLogger(name=__name__)


@app.task(ignore_result=True)
def task_process_document_file(document_file_id, user_id=None):
    DocumentFile = apps.get_model(
        app_label='documents', model_name='DocumentFile'
    )
    User = get_user_model()

    document_file = DocumentFile.objects.get(pk=document_file_id)

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    lock_id = 'task_process_document_file-%d' % document_file_id
    try:
        logger.debug('trying to acquire lock: %s', lock_id)
        # Acquire lock to avoid processing the same document file more
        # than once concurrently
        lock = LockingBackend.get_backend().acquire_lock(
            name=lock_id, timeout=LOCK_EXPIRE
        )
        logger.debug('acquired lock: %s', lock_id)
    except LockError:
        logger.debug('unable to obtain lock: %s' % lock_id)
    else:
        try:
            FileMetadataDriver.process_document_file(
                document_file=document_file, user=user
            )
        finally:
            lock.release()



from pathlib import Path
import shutil

from mayan.apps.documents.tests.literals import TEST_FILE_SMALL_PATH
from mayan.apps.sources.tests.mixins.base_mixins import SourceTestMixin
from mayan.apps.storage.utils import fs_cleanup, mkdtemp

from .literals import TEST_SOURCE_BACKEND_PATH_STORAGE


class SourceTestMixinStoredFile(SourceTestMixin):
    _test_source_backend_path = TEST_SOURCE_BACKEND_PATH_STORAGE
    _test_source_test_file_path = TEST_FILE_SMALL_PATH

    def setUp(self):
        self._test_source_temporary_folders = []
        super().setUp()

        self._test_source_path_test_file = Path(self._test_source_test_file_path)
        self._test_source_stored_file_list = []
        self._test_source_stored_test_file_list = []

    def tearDown(self):
        for test_source_temporary_folders in self._test_source_temporary_folders:
            fs_cleanup(filename=test_source_temporary_folders)

        super().tearDown()

    def _test_source_create_temporary_folder(self):
        self._test_source_temporary_folder = mkdtemp()
        self._test_source_temporary_folders.append(
            self._test_source_temporary_folder
        )

    def _test_source_pre_create(self, **kwargs):
        self._test_source_create_temporary_folder()

    def copy_test_source_file(self, source_path=None):
        path_destination = Path(
            self.get_test_source_storage_path()
        )

        source_path = source_path or self._test_source_path_test_file

        path_source = Path(source_path)

        shutil.copy(src=path_source, dst=path_destination)

        self._test_source_stored_test_file = path_destination / path_source.name
        self._test_source_stored_test_file_list.append(
            self._test_source_stored_test_file
        )

        self._test_source_stored_file_list = list(
            self._test_source.get_backend_instance().get_stored_file_list()
        )

        if self._test_source_stored_file_list:
            self._test_source_stored_file = self._test_source_stored_file_list[0]
        else:
            self._test_source_stored_file = None

    def get_test_source_storage_path(self):
        return self._test_source_temporary_folder

    def get_test_source_stored_file_list(self):
        backend_instance = self._test_source.get_backend_instance()

        return list(
            backend_instance.get_stored_file_list()
        )



from django.core.files import File

from mayan.apps.django_gpg.tests.literals import TEST_KEY_PRIVATE_PASSPHRASE

from ..models import DetachedSignature

from .literals import TEST_SIGNATURE_FILE_PATH


class DetachedSignatureAPIViewTestMixin:
    def _request_test_document_signature_detached_delete_view(self):
        return self.delete(
            viewname='rest_api:detachedsignature-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
                'detached_signature_id': self._test_document_file.signatures.first().pk
            }
        )

    def _request_test_document_signature_detached_detail_view(self):
        return self.get(
            viewname='rest_api:detachedsignature-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
                'detached_signature_id': self._test_document_file.signatures.first().pk
            }
        )

    def _request_test_document_signature_detached_list_view(self):
        return self.get(
            viewname='rest_api:document-file-signature-detached-list',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk
            }
        )

    def _request_test_document_signature_detached_sign_view(self):
        return self.post(
            viewname='rest_api:document-file-signature-detached-sign',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk
            }, data={
                'key': self._test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )

    def _request_test_document_signature_detached_upload_view(self):
        with open(file=TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            return self.post(
                viewname='rest_api:document-file-signature-detached-upload',
                kwargs={
                    'document_id': self._test_document.pk,
                    'document_file_id': self._test_document_file.pk
                }, data={'signature_file': file_object}
            )


class DetachedSignatureViewTestMixin:
    def _request_test_document_file_signature_create_view(self):
        return self.post(
            viewname='signatures:document_file_signature_detached_create',
            kwargs={
                'document_file_id': self._test_document_file.pk
            }, data={
                'key': self._test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )

    def _request_test_document_file_signature_detached_delete_view(self):
        return self.post(
            viewname='signatures:document_file_signature_detached_delete',
            kwargs={
                'signature_id': self._test_signature.pk
            }
        )

    def _request_test_document_file_signature_detached_download_view(self):
        return self.get(
            viewname='signatures:document_file_signature_detached_download',
            kwargs={'signature_id': self._test_signature.pk}
        )

    def _request_test_document_file_signature_detached_upload_view(self):
        with open(file=TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            return self.post(
                viewname='signatures:document_file_signature_detached_upload',
                kwargs={
                    'document_file_id': self._test_document.file_latest.pk
                }, data={'signature_file': file_object}
            )


class EmbeddedSignatureAPIViewTestMixin:
    def _request_test_document_signature_embedded_detail_view(self):
        return self.get(
            viewname='rest_api:embeddedsignature-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
                'embedded_signature_id': self._test_document_file.signatures.first().pk
            }
        )

    def _request_test_document_signature_embedded_list_view(self):
        return self.get(
            viewname='rest_api:document-file-signature-embedded-list',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk
            }
        )

    def _request_test_document_signature_embedded_sign_view(self):
        return self.post(
            viewname='rest_api:document-file-signature-embedded-sign',
            kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk
            }, data={
                'key': self._test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )


class EmbeddedSignatureViewTestMixin:
    def _request_test_document_file_embedded_signature_create_view(self):
        return self.post(
            viewname='signatures:document_file_signature_embedded_create',
            kwargs={
                'document_file_id': self._test_document_file.pk
            }, data={
                'key': self._test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )


class DetachedSignatureTestMixin:
    def _upload_test_detached_signature(self):
        with open(file=TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            self._test_signature = DetachedSignature.objects.create(
                document_file=self._test_document.file_latest,
                signature_file=File(file=file_object)
            )


class SignatureViewTestMixin:
    def _request_test_document_file_signature_detail_view(self):
        return self.get(
            viewname='signatures:document_file_signature_detail',
            kwargs={'signature_id': self._test_signature.pk}
        )

    def _request_test_document_file_signature_list_view(self, document):
        return self.get(
            viewname='signatures:document_file_signature_list',
            kwargs={
                'document_file_id': self._test_document.file_latest.pk
            }
        )


class SignatureToolsViewTestMixin:
    def _request_all_test_document_file_signature_refresh_view(self):
        return self.post(
            viewname='signatures:all_document_file_signature_refresh'
        )

    def _request_all_test_document_file_signature_verify_view(self):
        return self.post(
            viewname='signatures:all_document_file_signature_verify'
        )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0030_auto_20230505_1724')
    ]

    operations = [
        migrations.AlterField(
            model_name='workflowtransitionfield', name='required',
            field=models.BooleanField(
                default=False, help_text='Whether this field needs to be '
                'filled out or not to proceed.', verbose_name='Required'
            )
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0005_auto_20150729_2344'),
    ]

    operations = [
        migrations.AddField(
            model_name='metadatatype',
            name='parser',
            field=models.CharField(
                blank=True, help_text='The parser will reformat the value '
                'entered to conform to the expected format.', max_length=64,
                verbose_name='Parser',
                choices=[
                    (
                        b'metadata.validators.DateAndTimeValidator',
                        b'metadata.validators.DateAndTimeValidator'
                    ), (
                        b'metadata.validators.DateValidator',
                        b'metadata.validators.DateValidator'
                    ), (
                        b'metadata.validators.TimeValidator',
                        b'metadata.validators.TimeValidator'
                    )
                ]
            ), preserve_default=True,
        ),
        migrations.AlterField(
            model_name='metadatatype',
            name='validation',
            field=models.CharField(
                blank=True, help_text='The validator will reject data entry '
                'if the value entered does not conform to the expected '
                'format.', max_length=64, verbose_name='Validator',
                choices=[
                    (
                        b'metadata.validators.DateAndTimeValidator',
                        b'metadata.validators.DateAndTimeValidator'
                    ), (
                        b'metadata.validators.DateValidator',
                        b'metadata.validators.DateValidator'
                    ), (
                        b'metadata.validators.TimeValidator',
                        b'metadata.validators.TimeValidator'
                    )
                ]
            ), preserve_default=True,
        ),
    ]



import logging

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.source_compressed.source_backends.mixins import SourceBackendMixinCompressed
from mayan.apps.source_interactive.source_backend_actions.interactive_actions import (
    SourceBackendActionInteractiveDocumentUpload,
    SourceBackendActionInteractiveDocumentFileUpload
)
from mayan.apps.source_interactive.source_backends.mixins import SourceBackendMixinInteractive
from mayan.apps.sources.forms import WebFormUploadFormHTML5
from mayan.apps.sources.source_backends.base import SourceBackend
from mayan.apps.views.settings import setting_show_dropzone_submit_button

logger = logging.getLogger(name=__name__)


class SourceBackendWebForm(
    SourceBackendMixinCompressed, SourceBackendMixinInteractive,
    SourceBackend
):
    action_class_list = (
        SourceBackendActionInteractiveDocumentUpload,
        SourceBackendActionInteractiveDocumentFileUpload
    )
    label = _('Web form')
    upload_form_class = WebFormUploadFormHTML5

    def get_view_context(self, context, request):
        if setting_show_dropzone_submit_button.value:
            form_disable_submit = False
        else:
            form_disable_submit = True

        return {
            'subtemplates_list': [
                {
                    'context': {
                        'forms': context['forms'],
                        'form_action': '{}?{}'.format(
                            reverse(
                                viewname=request.resolver_match.view_name,
                                kwargs=request.resolver_match.kwargs
                            ), request.META['QUERY_STRING']
                        ),
                        'form_disable_submit': form_disable_submit,
                        'form_id': 'html5upload',
                        'is_multipart': True
                    },
                    'name': 'appearance/generic_multiform_subtemplate.html'
                }
            ]
        }



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0075_delete_duplicateddocumentold'),
        ('duplicates', '0007_auto_20201130_0828')
    ]

    operations = [
        migrations.CreateModel(
            name='DuplicateSourceDocument',
            fields=[],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': []
            },
            bases=('documents.document',)
        ),
        migrations.CreateModel(
            name='DuplicateTargetDocument',
            fields=[],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': []
            },
            bases=('documents.document',)
        ),
        migrations.AlterField(
            model_name='duplicatebackendentry',
            name='document',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='duplicates', to='documents.Document',
                verbose_name='Document'
            )
        ),
        migrations.AlterUniqueTogether(
            name='duplicatebackendentry',
            unique_together={
                ('stored_backend', 'document')
            }
        )
    ]



import os

from django.apps import apps
from django.core.exceptions import AppRegistryNotReady
from django.utils.translation.trans_real import DjangoTranslation


def patchDjangoTranslation():
    """
    Patch Django to prioritize the Mayan app translations over its own.
    Fixes GitLab issue #734 for Django 1.11
    Needs to be updated for future Django version unless this is fixed
    upstream.
    """
    def _add_installed_apps_translations_new(self):
        """Merge translations from each installed app."""
        try:
            non_mayan_app_configs = [
                app for app in apps.get_app_configs() if not app.name.startswith('mayan.')
            ]

            mayan_app_configs = [
                app for app in apps.get_app_configs() if app.name.startswith('mayan.')
            ]
            mayan_app_configs.extend(non_mayan_app_configs)
            app_configs = mayan_app_configs
        except AppRegistryNotReady:
            raise AppRegistryNotReady(
                "The translation infrastructure cannot be initialized before the "
                "apps registry is ready. Check that you don't make non-lazy "
                "gettext calls at import time.")
        for app_config in app_configs:
            localedir = os.path.join(app_config.path, 'locale')
            if os.path.exists(localedir):
                translation = self._new_gnu_trans(localedir)
                self.merge(translation)

    DjangoTranslation._add_installed_apps_translations = _add_installed_apps_translations_new
